@@ -1,5 +1,5 @@
 /*
- * $Id: display.c,v 1.122.2.1 2003/07/23 15:45:09 tal197 Exp $
+ * $Id: display.c,v 1.124 2003/08/02 13:01:07 kerofin Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -706,8 +706,8 @@ void display_update_view(FilerWindow *filer_window,
 		view->image = NULL;
 	}
 
-	if (filer_window->show_thumbs && item->base_type == TYPE_FILE &&
-		strcmp(item->mime_type->media_type, "image") == 0)
+	if (filer_window->show_thumbs && item->base_type == TYPE_FILE /*&&
+									strcmp(item->mime_type->media_type, "image") == 0*/)
 	{
 		const guchar    *path;
 
