@@ -1,5 +1,5 @@
 /*
- * $Id: dnd.c,v 1.124 2003/03/27 12:08:55 tal197 Exp $
+ * $Id: dnd.c,v 1.129 2004/01/21 20:03:15 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -90,7 +90,7 @@ static void got_data_raw(GtkWidget 		*widget,
 			guint32             	time);
 static void got_uri_list(GtkWidget 		*widget,
 			 GdkDragContext 	*context,
-			 GtkSelectionData 	*selection_data,
+			 const char	 	*selection_data,
 			 guint32             	time);
 static gboolean drag_drop(GtkWidget 	  *widget,
 			  GdkDragContext  *context,
@@ -137,6 +137,7 @@ const char *drop_dest_bookmark = "drop_dest_bookmark";	/* Add to bookmarks */
 GdkAtom XdndDirectSave0;
 GdkAtom xa_text_plain;
 GdkAtom text_uri_list;
+GdkAtom text_x_moz_url;
 GdkAtom application_octet_stream;
 GdkAtom xa_string; /* Not actually used for DnD, but the others are here! */
 
@@ -153,6 +154,7 @@ void dnd_init(void)
 	XdndDirectSave0 = gdk_atom_intern("XdndDirectSave0", FALSE);
 	xa_text_plain = gdk_atom_intern("text/plain", FALSE);
 	text_uri_list = gdk_atom_intern("text/uri-list", FALSE);
+	text_x_moz_url = gdk_atom_intern("text/x-moz-url", FALSE);
 	application_octet_stream = gdk_atom_intern("application/octet-stream",
 			FALSE);
 	xa_string = gdk_atom_intern("STRING", FALSE);
@@ -263,7 +265,7 @@ void drag_one_item(GtkWidget		*widget,
 		   DirItem		*item,
 		   MaskedPixmap		*image)
 {
-	guchar		*uri;
+	guchar		*uri, *tmp;
 	GdkDragContext 	*context;
 	GdkDragAction	actions;
 	GtkTargetList   *target_list;
@@ -313,8 +315,10 @@ void drag_one_item(GtkWidget		*widget,
 
 	g_dataset_set_data_full(context, "full_path",
 			g_strdup(full_path), g_free);
-	uri = g_strconcat("file://", our_host_name_for_dnd(),
-			full_path, "\r\n", NULL);
+	tmp = encode_path_as_uri(full_path);
+	uri = g_strconcat(tmp, "\r\n", NULL);
+	/*printf("%s\n", tmp);*/
+	g_free(tmp);
 	g_dataset_set_data_full(context, "uri_list", uri, g_free);
 
 	g_return_if_fail(image != NULL);
@@ -338,7 +342,7 @@ static gchar *uri_list_to_utf8(const char *uri_list)
 	for (next_uri = uris; next_uri; next_uri = next_uri->next)
 	{
 		char *uri = (char *) next_uri->data;
-		const char *local;
+		char *local;
 
 		local = get_local_path(uri);
 
@@ -346,7 +350,10 @@ static gchar *uri_list_to_utf8(const char *uri_list)
 			g_string_append_c(new, ' ');
 
 		if (local)
+		{
 			g_string_append(new, local);
+			g_free(local);
+		}
 		else
 			g_warning("Not local!\n");
 
@@ -433,6 +440,7 @@ void make_drop_target(GtkWidget *widget, GtkDestDefaults defaults)
 	GtkTargetEntry 	target_table[] =
 	{
 		{"text/uri-list", 0, TARGET_URI_LIST},
+		{"text/x-moz-url", 0, TARGET_MOZ_URL},
 		{"XdndDirectSave0", 0, TARGET_XDS},
 		{"application/octet-stream", 0, TARGET_RAW},
 	};
@@ -504,12 +512,14 @@ const guchar *dnd_motion_item(GdkDragContext *context, DirItem **item_p)
 	{
 		/* A normal directory */
 		if (provides(context, text_uri_list) ||
+				provides(context, text_x_moz_url) ||
 				provides(context, XdndDirectSave0))
 			return drop_dest_dir;
 	}
 	else
 	{
 		if (provides(context, text_uri_list) ||
+				provides(context, text_x_moz_url) ||
 				provides(context, application_octet_stream))
 			return drop_dest_prog;
 	}
@@ -567,15 +577,12 @@ static gboolean drag_drop(GtkWidget 	  *widget,
 			}
 			else
 			{
-				GString	*uri;
+				gchar	*uri;
 
-				uri = g_string_new(NULL);
-				g_string_printf(uri, "file://%s%s",
-						our_host_name_for_dnd(),
-						make_path(dest_path,
+				uri = encode_path_as_uri(make_path(dest_path,
 							  leafname));
-				set_xds_prop(context, uri->str);
-				g_string_free(uri, TRUE);
+				set_xds_prop(context, uri);
+				g_free(uri);
 
 				target = XdndDirectSave0;
 				g_dataset_set_data_full(context, "leafname",
@@ -590,6 +597,8 @@ static gboolean drag_drop(GtkWidget 	  *widget,
 	}
 	else if (provides(context, text_uri_list))
 		target = text_uri_list;
+	else if (provides(context, text_x_moz_url))
+		target = text_x_moz_url;
 	else if (provides(context, application_octet_stream))
 		target = application_octet_stream;
 	else
@@ -647,13 +656,16 @@ static void desktop_drag_data_received(GtkWidget      	*widget,
 
 	for (next = uris; next; next = next->next)
 	{
-		const guchar	*path;
+		guchar	*path;
 
 		path = get_local_path((gchar *) next->data);
+		/*printf("%s -> %s\n", (char *) next->data,
+		  path? path: "NULL");*/
 		if (path)
 		{
 			pinboard_pin(path, NULL, x, y, NULL);
 			x += 64;
+			g_free(path);
 		}
 
 		g_free(next->data);
@@ -663,6 +675,36 @@ static void desktop_drag_data_received(GtkWidget      	*widget,
 		g_list_free(uris);
 }
 
+/* Convert Mozilla's text/x-moz-uri into a text/uri-list */
+static void got_moz_uri(GtkWidget 		*widget,
+			GdkDragContext 		*context,
+			GtkSelectionData	*selection_data,
+			guint32        		time)
+{
+	gchar *utf8, *uri_list, *eol;
+
+	utf8 = g_utf16_to_utf8((gunichar2 *) selection_data->data,
+			(glong) selection_data->length,
+			NULL, NULL, NULL);
+
+	eol = utf8 ? strchr(utf8, '\n') : NULL;
+	if (!eol)
+	{
+		delayed_error("Invalid UTF16 from text/x-moz-url target");
+		g_free(utf8);
+		gtk_drag_finish(context, FALSE, FALSE, time);
+		return;
+	}
+
+	*eol = '\0';
+	uri_list = g_strconcat(utf8, "\r\n", NULL);
+	g_free(utf8);
+
+	got_uri_list(widget, context, uri_list, time);
+
+	g_free(uri_list);
+}
+
 /* Called when some data arrives from the remote app (which we asked for
  * in drag_drop).
  */
@@ -692,7 +734,11 @@ static void drag_data_received(GtkWidget      	*widget,
 			got_data_raw(widget, context, selection_data, time);
 			break;
 		case TARGET_URI_LIST:
-			got_uri_list(widget, context, selection_data, time);
+			got_uri_list(widget, context, selection_data->data,
+					time);
+			break;
+		case TARGET_MOZ_URL:
+			got_moz_uri(widget, context, selection_data, time);
 			break;
 		default:
 			gtk_drag_finish(context, FALSE, FALSE, time);
@@ -825,6 +871,16 @@ static void got_data_raw(GtkWidget 		*widget,
 		gtk_drag_finish(context, TRUE, FALSE, time);    /* Success! */
 }
 
+static gboolean uri_is_local(const char *uri)
+{
+	char *path;
+	path = get_local_path(uri);
+	if (!path)
+		return FALSE;
+	g_free(path);
+	return TRUE;
+}
+
 /* We've got a list of URIs from somewhere (probably another filer window).
  * If the files are on the local machine then try to copy them ourselves,
  * otherwise, if there was only one file and application/octet-stream was
@@ -832,7 +888,7 @@ static void got_data_raw(GtkWidget 		*widget,
  */
 static void got_uri_list(GtkWidget 		*widget,
 			 GdkDragContext 	*context,
-			 GtkSelectionData 	*selection_data,
+			 const char 		*selection_data,
 			 guint32             	time)
 {
 	GList		*uri_list;
@@ -845,7 +901,7 @@ static void got_uri_list(GtkWidget 		*widget,
 	dest_path = g_dataset_get_data(context, "drop_dest_path");
 	type = g_dataset_get_data(context, "drop_dest_type");
 
-	uri_list = uri_list_to_glist(selection_data->data);
+	uri_list = uri_list_to_glist(selection_data);
 
 	if (type == drop_dest_bookmark)
 	{
@@ -863,7 +919,7 @@ static void got_uri_list(GtkWidget 		*widget,
 		error = _("No URIs in the text/uri-list (nothing to do!)");
 	else if (context->action != GDK_ACTION_ASK && type == drop_dest_prog)
 		run_with_files(dest_path, uri_list);
-	else if ((!uri_list->next) && (!get_local_path(uri_list->data)))
+	else if ((!uri_list->next) && !uri_is_local(uri_list->data))
 	{
 		/* There is one URI in the list, and it's not on the local
 		 * machine. Get it via the X server if possible.
@@ -878,7 +934,7 @@ static void got_uri_list(GtkWidget 		*widget,
 			else
 				leaf = uri_list->data;
 			g_dataset_set_data_full(context, "leafname",
-					g_strdup(leaf), g_free);
+					unescape_uri(leaf), g_free);
 			gtk_drag_get_data(widget, context,
 					application_octet_stream, time);
 			send_reply = FALSE;
@@ -889,7 +945,7 @@ static void got_uri_list(GtkWidget 		*widget,
 	}
 	else
 	{
-		GList		*local_paths = NULL;
+		GList *local_paths = NULL;
 
 		/* Either one local URI, or a list. If everything in the list
 		 * isn't local then we are stuck.
@@ -897,13 +953,15 @@ static void got_uri_list(GtkWidget 		*widget,
 
 		for (next_uri = uri_list; next_uri; next_uri = next_uri->next)
 		{
-			const char *path;
+			char *path;
 
 			path = get_local_path((char *) next_uri->data);
+			/*printf("%s -> %s\n", (char *) next_uri->data,
+			  path? path: "NULL");*/
 
-			if (path)
+			if (path) 
 				local_paths = g_list_append(local_paths,
-								g_strdup(path));
+								path);
 			else
 				error = _("Some of these files are on a "
 					"different machine - they will be "
