@@ -1,5 +1,5 @@
 /*
- * $Id: filer.c,v 1.329 2003/05/27 13:03:16 tal197 Exp $
+ * $Id: filer.c,v 1.339 2003/10/26 16:14:28 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -82,6 +82,8 @@ FilerWindow 	*window_with_focus = NULL;
 
 GList		*all_filer_windows = NULL;
 
+static GHashTable *window_with_id = NULL;
+
 static FilerWindow *window_with_primary = NULL;
 
 /* Static prototypes */
@@ -151,6 +153,9 @@ void filer_init(void)
 	busy_cursor = gdk_cursor_new(GDK_WATCH);
 	crosshair = gdk_cursor_new(GDK_CROSSHAIR);
 
+	window_with_id = g_hash_table_new_full(g_str_hash, g_str_equal,
+					       NULL, NULL);
+
 	/* Is the display on the local machine, or are we being
 	 * run remotely? See filer_set_title().
 	 */
@@ -435,6 +440,8 @@ static void filer_window_destroyed(GtkWidget *widget, FilerWindow *filer_window)
 
 	tooltip_show(NULL);
 
+	filer_set_id(filer_window, NULL);
+
 	g_free(filer_window->auto_select);
 	g_free(filer_window->real_path);
 	g_free(filer_window->sym_path);
@@ -785,12 +792,12 @@ static void group_save(FilerWindow *filer_window, char *name)
 			NULL, "group", NULL);
 	xmlSetProp(group, "name", name);
 
-	xmlNewChild(group, NULL, "directory", filer_window->sym_path);
+	xmlNewTextChild(group, NULL, "directory", filer_window->sym_path);
 
 	view_get_iter(filer_window->view, &iter, VIEW_ITER_SELECTED);
 
 	while ((item = iter.next(&iter)))
-		xmlNewChild(group, NULL, "item", item->leafname);
+		xmlNewTextChild(group, NULL, "item", item->leafname);
 
 	save_path = choices_find_path_save("Groups.xml", PROJECT, TRUE);
 	if (save_path)
@@ -1184,6 +1191,7 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 	filer_window->view = NULL;
 	filer_window->scrollbar = NULL;
 	filer_window->auto_scroll = -1;
+	filer_window->window_id = NULL;
 
 	tidy_sympath(filer_window->sym_path);
 
@@ -1205,6 +1213,7 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 	filer_window->flags = (FilerFlags) 0;
 	filer_window->details_type = DETAILS_TIMES;
 	filer_window->display_style = UNKNOWN_STYLE;
+	filer_window->display_style_wanted = UNKNOWN_STYLE;
 	filer_window->thumb_queue = NULL;
 	filer_window->max_thumbs = 0;
 	filer_window->sort_type = -1;
@@ -1574,6 +1583,11 @@ void filer_check_mounted(const char *real_path)
 	int	len;
 	gboolean resize = o_filer_auto_resize.int_value == RESIZE_ALWAYS;
 
+	/* DOS disks, etc, often don't change the mtime of the root directory
+	 * on modification, so force a refresh now.
+	 */
+	g_fscache_update(dir_cache, real_path);
+
 	len = strlen(real_path);
 
 	while (next)
@@ -1657,6 +1671,31 @@ gboolean filer_exists(FilerWindow *filer_window)
 	return FALSE;
 }
 
+FilerWindow *filer_get_by_id(const char *id)
+{
+	return g_hash_table_lookup(window_with_id, id);
+}
+
+void filer_set_id(FilerWindow *filer_window, const char *id)
+{
+	g_return_if_fail(filer_window != NULL);
+
+	if (filer_window->window_id)
+	{
+		g_hash_table_remove(window_with_id, filer_window->window_id);
+		g_free(filer_window->window_id);
+		filer_window->window_id = NULL;
+	}
+
+	if (id)
+	{
+		filer_window->window_id = g_strdup(id);
+		g_hash_table_insert(window_with_id,
+				filer_window->window_id,
+				filer_window);
+	}
+}
+
 /* Make sure the window title is up-to-date */
 void filer_set_title(FilerWindow *filer_window)
 {
@@ -1892,8 +1931,8 @@ void filer_create_thumbs(FilerWindow *filer_window)
 		 if (item->base_type != TYPE_FILE)
 			 continue;
 
-		 if (strcmp(item->mime_type->media_type, "image") != 0)
-			 continue;
+		 /*if (strcmp(item->mime_type->media_type, "image") != 0)
+		   continue;*/
 
 		path = make_path(filer_window->real_path, item->leafname);
 
@@ -1998,18 +2037,21 @@ static guchar *filer_create_uri_list(FilerWindow *filer_window)
 
 	string = g_string_new(NULL);
 
-	leader = g_string_new("file://");
-	g_string_append(leader, our_host_name_for_dnd());
-	g_string_append(leader, filer_window->sym_path);
+	leader = g_string_new(filer_window->sym_path);
 	if (leader->str[leader->len - 1] != '/')
 		g_string_append_c(leader, '/');
 
 	view_get_iter(filer_window->view, &iter, VIEW_ITER_SELECTED);
 	while ((item = iter.next(&iter)))
 	{
-		g_string_append(string, leader->str);
-		g_string_append(string, item->leafname);
+		gchar *uri, *path;
+		
+		path = g_strconcat(leader->str, item->leafname, NULL);
+		uri = encode_path_as_uri(path);
+		g_string_append(string, uri);
 		g_string_append(string, "\r\n");
+		g_free(path);
+		g_free(uri);
 	}
 
 	g_string_free(leader, TRUE);
@@ -2473,3 +2515,36 @@ void filer_set_autoscroll(FilerWindow *filer_window, gboolean auto_scroll)
 		filer_window->auto_scroll = -1;
 	}
 }
+
+#define ZERO_MNT "/uri/0install"
+
+static void refresh_done(FilerWindow *filer_window)
+{
+	if (filer_exists(filer_window))
+		filer_update_dir(filer_window, TRUE);
+}
+
+void filer_refresh(FilerWindow *filer_window)
+{
+	if (!strncmp(ZERO_MNT "/", filer_window->real_path, sizeof(ZERO_MNT)))
+	{
+		/* Try to run 0refresh */
+		gint pid;
+		gchar *argv[] = {"0refresh", NULL, NULL};
+		const char *host = filer_window->real_path + sizeof(ZERO_MNT);
+		const char *slash;
+
+		slash = strchr(host, '/');
+		if (slash)
+			argv[1] = g_strndup(host, slash - host);
+		else
+			argv[1] = g_strdup(host);
+		pid = rox_spawn(filer_window->real_path, (const char **) argv);
+		g_free(argv[1]);
+		if (pid)
+			on_child_death(pid, (CallbackFn) refresh_done,
+					filer_window);
+	}
+	
+	full_refresh();
+}
