@@ -1,5 +1,5 @@
 /*
- * $Id: find.c,v 1.22 2003/01/19 19:29:04 tal197 Exp $
+ * $Id: find.c,v 1.23 2003/10/26 14:52:35 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -682,7 +682,7 @@ static FindCondition *parse_dash(const gchar **expression)
 	if (NEXT != '-')
 		return NULL;
 
-	while (exp[i] && !isspace(exp[i]))
+	while (exp[i] && !g_ascii_isspace(exp[i]))
 	{
 		switch (exp[i])
 		{
@@ -998,7 +998,7 @@ static gboolean match(const gchar **expression, const gchar *word)
 	if (g_strncasecmp(*expression, word, len))
 		return FALSE;
 
-	if (isalpha(*(*expression + len)))
+	if (g_ascii_isalpha(*(*expression + len)))
 		return FALSE;
 
 	(*expression) += len;
