@@ -1,5 +1,5 @@
 /*
- * $Id: gui_support.c,v 1.107 2003/04/25 10:21:12 tal197 Exp $
+ * $Id: gui_support.c,v 1.111 2003/10/07 14:01:55 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -242,8 +242,6 @@ void make_panel_window(GtkWidget *widget)
 	gdk_property_change(window, xa_net_state, xa_atom, 32,
 			GDK_PROP_MODE_APPEND, (guchar *) state_list, 3);
 
-	g_return_if_fail(window != NULL);
-
 	gdk_property_change(window, xa_hints, xa_hints, 32,
 			GDK_PROP_MODE_REPLACE, (guchar *) wm_hints_values,
 			sizeof(wm_hints_values) / sizeof(gint32));
@@ -511,27 +509,28 @@ void destroy_on_idle(GtkWidget *widget)
 }
 
 /* Spawn a child process (as spawn_full), and report errors.
- * TRUE on success.
+ * Returns the child's PID on succes, or 0 on failure.
  */
-gboolean rox_spawn(const gchar *dir, const gchar **argv)
+gint rox_spawn(const gchar *dir, const gchar **argv)
 {
 	GError	*error = NULL;
+	gint pid = 0;
 
 	if (!g_spawn_async_with_pipes(dir, (gchar **) argv, NULL,
 			G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_STDOUT_TO_DEV_NULL |
 			G_SPAWN_SEARCH_PATH,
 			NULL, NULL,		/* Child setup fn */
-			NULL,			/* Child PID */
+			&pid,			/* Child PID */
 			NULL, NULL, NULL,	/* Standard pipes */
 			&error))
 	{
 		delayed_error("%s", error ? error->message : "(null)");
 		g_error_free(error);
 
-		return FALSE;
+		return 0;
 	}
 
-	return TRUE;
+	return pid;
 }
 
 GtkWidget *button_new_image_text(GtkWidget *image, const char *message)
@@ -929,7 +928,7 @@ gint radios_get_value(Radios *radios)
 	return -1;
 }
 
-/* Convert a list of URIs into a list of strings.
+/* Convert a list of URIs as a string into a GList of URIs.
  * Lines beginning with # are skipped.
  * The text block passed in is zero terminated (after the final CRLF)
  */
@@ -957,7 +956,10 @@ GList *uri_list_to_glist(const char *uri_list)
 
 		if (length && uri_list[0] != '#')
 		{
-			uri = g_strndup(uri_list, length);
+			char *tmp;
+			tmp = g_strndup(uri_list, length);
+			uri = unescape_uri(tmp);
+			g_free(tmp);
 			list = g_list_append(list, uri);
 		}
 
