@@ -1,5 +1,5 @@
 /*
- * $Id: icon.c,v 1.67 2003/03/07 13:23:57 tal197 Exp $
+ * $Id: icon.c,v 1.78 2004/01/14 13:47:49 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -117,11 +117,11 @@ static void grab_key(Icon *icon);
 static void parseKeyString(MyKey *key, const char *str);
 static void icon_wink(Icon *icon);
 static void initModifiers(void);
+static void create_menu(void);
 
 enum {
 	ACTION_SHIFT,
-	ACTION_HELP,
-	ACTION_INFO,
+	ACTION_PROPERTIES,
 	ACTION_RUN_ACTION,
 	ACTION_SET_ICON,
 	ACTION_EDIT,
@@ -132,7 +132,7 @@ enum {
 #define N_(x) x
 static GtkItemFactoryEntry menu_def[] = {
 {N_("ROX-Filer"),		NULL, NULL, 0, "<Branch>"},
-{">" N_("About ROX-Filer..."),	NULL, menu_rox_help, HELP_ABOUT, NULL},
+{">" N_("About ROX-Filer..."),	NULL, menu_rox_help, HELP_ABOUT, "<StockItem>", GTK_STOCK_DIALOG_INFO},
 {">" N_("Show Help Files"),	NULL, menu_rox_help, HELP_DIR, "<StockItem>", GTK_STOCK_HELP},
 {">" N_("Manual"),		NULL, menu_rox_help, HELP_MANUAL, NULL},
 {">",				NULL, NULL, 0, "<Separator>"},
@@ -140,13 +140,12 @@ static GtkItemFactoryEntry menu_def[] = {
 {">" N_("Home Directory"),	NULL, open_home, 0, "<StockItem>", GTK_STOCK_HOME},
 {N_("File"),			NULL, NULL, 0, "<Branch>"},
 {">" N_("Shift Open"),   	NULL, file_op, ACTION_SHIFT, NULL},
-{">" N_("Help"),    		NULL, file_op, ACTION_HELP, "<StockItem>", GTK_STOCK_HELP},
-{">" N_("Info"),    		NULL, file_op, ACTION_INFO, "<StockItem>", GTK_STOCK_DIALOG_INFO},
+{">" N_("Properties"),    	NULL, file_op, ACTION_PROPERTIES, "<StockItem>", GTK_STOCK_PROPERTIES},
 {">" N_("Set Run Action..."),	NULL, file_op, ACTION_RUN_ACTION, "<StockItem>", GTK_STOCK_EXECUTE},
 {">" N_("Set Icon..."),		NULL, file_op, ACTION_SET_ICON, NULL},
-{N_("Edit Item"),  		NULL, file_op, ACTION_EDIT, NULL},
-{N_("Show Location"),  		NULL, file_op, ACTION_LOCATION, NULL},
-{N_("Remove Item(s)"),		NULL, remove_items, 0, NULL},
+{N_("Edit Item"),  		NULL, file_op, ACTION_EDIT, "<StockItem>", GTK_STOCK_PROPERTIES},
+{N_("Show Location"),  		NULL, file_op, ACTION_LOCATION, "<StockItem>", GTK_STOCK_JUMP_TO},
+{N_("Remove Item(s)"),		NULL, remove_items, 0, "<StockItem>", GTK_STOCK_REMOVE},
 {"",				NULL, NULL, 0, "<Separator>"},
 {N_("Backdrop..."),		NULL, set_backdrop, 0, NULL},
 };
@@ -240,6 +239,9 @@ void icon_prepare_menu(Icon *icon, gboolean pinboard)
 
 	menu_icon = icon;
 
+	if (!icon_menu)
+		create_menu();
+
 	menu_set_hidden(icon_menu, !pinboard, 5, 2);
 
 	/* Shade Remove Item(s) unless there is a selection */
@@ -256,11 +258,11 @@ void icon_prepare_menu(Icon *icon, gboolean pinboard)
 		guchar *tmp;
 
 		menu_set_items_shaded(icon_menu, FALSE, 1, 3);
-		menu_set_items_shaded(icon_file_menu, FALSE, 0, 6);
+		menu_set_items_shaded(icon_file_menu, FALSE, 0, 5);
 		if (!can_set_run_action(icon->item))
-			menu_set_items_shaded(icon_file_menu, TRUE, 3, 1);
+			menu_set_items_shaded(icon_file_menu, TRUE, 2, 1);
 
-		tmp = g_strdup_printf("%s '%s'",
+		tmp = g_strdup_printf(_("%s '%s'"),
 				basetype_name(icon->item),
 				icon->item->leafname);
 		gtk_label_set_text(GTK_LABEL(icon_file_item), tmp);
@@ -272,7 +274,7 @@ void icon_prepare_menu(Icon *icon, gboolean pinboard)
 	else
 	{
 		menu_set_items_shaded(icon_menu, TRUE, 1, 3);
-		menu_set_items_shaded(icon_file_menu, TRUE, 0, 6);
+		menu_set_items_shaded(icon_file_menu, TRUE, 0, 5);
 		gtk_label_set_text(GTK_LABEL(icon_file_item), _("Nothing"));
 	}
 }
@@ -334,22 +336,21 @@ gchar *icon_create_uri_list(void)
 {
 	GString	*tmp;
 	guchar	*retval;
-	guchar	*leader;
 	GList	*next;
 
 	tmp = g_string_new(NULL);
-	leader = g_strdup_printf("file://%s", our_host_name_for_dnd());
 
 	for (next = icon_selection; next; next = next->next)
 	{
 		Icon *icon = (Icon *) next->data;
+		gchar *uri;
 
-		g_string_append(tmp, leader);
-		g_string_append(tmp, icon->path);
+		uri = encode_path_as_uri(icon->path);
+		g_string_append(tmp, uri);
+		g_free(uri);
 		g_string_append(tmp, "\r\n");
 	}
 
-	g_free(leader);
 	retval = tmp->str;
 	g_string_free(tmp, FALSE);
 	
@@ -401,13 +402,16 @@ void icon_set_path(Icon *icon, const char *pathname, const char *name)
 
 	if (pathname)
 	{
-		icon->src_path = g_strdup(pathname);
-		icon->path = expand_path(pathname);
+		if (g_utf8_validate(pathname, -1, NULL))
+			icon->src_path = g_strdup(pathname);
+		else
+			icon->src_path = to_utf8(pathname);
+		icon->path = expand_path(icon->src_path);
 
 		icon_hash_path(icon);
 
 		if (!name)
-			name = g_basename(pathname);
+			name = g_basename(icon->src_path);
 
 		icon->item = diritem_new(name);
 		diritem_restat(icon->path, icon->item, NULL);
@@ -561,7 +565,7 @@ static void selection_get(GtkClipboard	*primary,
 
 static void set_backdrop(gpointer data, guint action, GtkWidget *widget)
 {
-	pinboard_set_backdrop();
+	pinboard_set_backdrop_box();
 }
 
 static void remove_items(gpointer data, guint action, GtkWidget *widget)
@@ -604,10 +608,7 @@ static void file_op(gpointer data, guint action, GtkWidget *widget)
 		case ACTION_LOCATION:
 			open_to_show(menu_icon->path);
 			break;
-		case ACTION_HELP:
-			show_item_help(menu_icon->path, menu_icon->item);
-			break;
-		case ACTION_INFO:
+		case ACTION_PROPERTIES:
 			infobox_new(menu_icon->path);
 			break;
 		case ACTION_RUN_ACTION:
@@ -855,9 +856,6 @@ static void icon_finialize(GObject *object)
 
 static void icon_class_init(gpointer gclass, gpointer data)
 {
-	guchar		*tmp;
-	GList		*items;
-	GtkItemFactory	*item_factory;
 	GObjectClass *object = (GObjectClass *) gclass;
 	IconClass *icon = (IconClass *) gclass;
 
@@ -895,28 +893,6 @@ static void icon_class_init(gpointer gclass, gpointer data)
 			G_TYPE_NONE, 0);
 
 	icons_hash = g_hash_table_new(g_str_hash, g_str_equal);
-
-	item_factory = menu_create(menu_def,
-				sizeof(menu_def) / sizeof(*menu_def),
-				 "<icon>", NULL);
-
-	tmp = g_strconcat("<icon>/", _("File"), NULL);
-	icon_menu = gtk_item_factory_get_widget(item_factory, "<icon>");
-	icon_file_menu = gtk_item_factory_get_widget(item_factory, tmp);
-	g_free(tmp);
-
-	/* File '' label... */
-	items = gtk_container_get_children(GTK_CONTAINER(icon_menu));
-	icon_file_item = GTK_BIN(g_list_nth(items, 1)->data)->child;
-	g_list_free(items);
-
-	/* Shift Open... label */
-	items = gtk_container_get_children(GTK_CONTAINER(icon_file_menu));
-	file_shift_item = GTK_BIN(items->data)->child;
-	g_list_free(items);
-
-	g_signal_connect(icon_menu, "unmap_event",
-			G_CALLBACK(menu_closed), NULL);
 }
 
 static void icon_init(GTypeInstance *object, gpointer gclass)
@@ -1150,7 +1126,7 @@ static GdkFilterReturn filter_keys(GdkXEvent *xevent,
 	GList *next;
 	XKeyEvent *kev = (XKeyEvent *) xevent;
 	guint state;
-	
+
 	if (kev->type != KeyPress)
 		return GDK_FILTER_CONTINUE;
 
@@ -1214,19 +1190,14 @@ static gboolean grabKey(MyKey *key)
 
 	GRAB(key, 0);
 
-	if (key->modifier != AnyModifier && key->modifier != 0)
-	{
-		/* Here we grab all combinations of well known
-		 * modifiers.
-		 */
-		GRAB(key, ScrollLockMask);
-		GRAB(key, NumLockMask);
-		GRAB(key, CapsLockMask);
-		GRAB(key, ScrollLockMask | NumLockMask);
-		GRAB(key, ScrollLockMask | CapsLockMask);
-		GRAB(key, CapsLockMask | NumLockMask);
-		GRAB(key, ScrollLockMask | CapsLockMask | NumLockMask);
-	}
+	/* Here we grab all combinations of well known modifiers */
+	GRAB(key, ScrollLockMask);
+	GRAB(key, NumLockMask);
+	GRAB(key, CapsLockMask);
+	GRAB(key, ScrollLockMask | NumLockMask);
+	GRAB(key, ScrollLockMask | CapsLockMask);
+	GRAB(key, CapsLockMask | NumLockMask);
+	GRAB(key, ScrollLockMask | CapsLockMask | NumLockMask);
 
 	gdk_flush();
 	return gdk_error_trap_pop() == Success;
@@ -1241,16 +1212,13 @@ static gboolean ungrabKey(MyKey *key)
 
 	UNGRAB(key, 0);
 
-	if (key->modifier != AnyModifier && key->modifier != 0)
-	{
-		UNGRAB(key, ScrollLockMask);
-		UNGRAB(key, NumLockMask);
-		UNGRAB(key, CapsLockMask);
-		UNGRAB(key, ScrollLockMask | NumLockMask);
-		UNGRAB(key, ScrollLockMask | CapsLockMask);
-		UNGRAB(key, CapsLockMask | NumLockMask);
-		UNGRAB(key, ScrollLockMask | CapsLockMask | NumLockMask);
-	}
+	UNGRAB(key, ScrollLockMask);
+	UNGRAB(key, NumLockMask);
+	UNGRAB(key, CapsLockMask);
+	UNGRAB(key, ScrollLockMask | NumLockMask);
+	UNGRAB(key, ScrollLockMask | CapsLockMask);
+	UNGRAB(key, CapsLockMask | NumLockMask);
+	UNGRAB(key, ScrollLockMask | CapsLockMask | NumLockMask);
 
 	gdk_flush();
 	return gdk_error_trap_pop() == Success;
@@ -1329,3 +1297,36 @@ static void icon_wink(Icon *icon)
 
 	iclass->wink(icon);
 }
+
+/* Sets icon_menu */
+static void create_menu(void)
+{
+	GList		*items;
+	guchar		*tmp;
+	GtkItemFactory	*item_factory;
+
+	g_return_if_fail(icon_menu == NULL);
+
+	item_factory = menu_create(menu_def,
+				sizeof(menu_def) / sizeof(*menu_def),
+				 "<icon>", NULL);
+
+	tmp = g_strconcat("<icon>/", _("File"), NULL);
+	icon_menu = gtk_item_factory_get_widget(item_factory, "<icon>");
+	icon_file_menu = gtk_item_factory_get_widget(item_factory, tmp);
+	g_free(tmp);
+
+	/* File '' label... */
+	items = gtk_container_get_children(GTK_CONTAINER(icon_menu));
+	icon_file_item = GTK_BIN(g_list_nth(items, 1)->data)->child;
+	g_list_free(items);
+
+	/* Shift Open... label */
+	items = gtk_container_get_children(GTK_CONTAINER(icon_file_menu));
+	file_shift_item = GTK_BIN(items->data)->child;
+	g_list_free(items);
+
+	g_signal_connect(icon_menu, "unmap_event",
+			G_CALLBACK(menu_closed), NULL);
+}
+
