@@ -1,5 +1,5 @@
 /*
- * $Id: infobox.c,v 1.46 2003/02/27 11:50:40 tal197 Exp $
+ * $Id: infobox.c,v 1.57 2004/01/24 19:00:22 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/param.h>
+#include <signal.h>
 #include <libxml/parser.h>
 
 #include <gtk/gtk.h>
@@ -41,7 +42,7 @@
 #include "infobox.h"
 #include "appinfo.h"
 #include "dnd.h"	/* For xa_string */
-#include "run.h"	/* For show_item_help() */
+#include "run.h"	/* For show_help_files() */
 #include "xml.h"
 #include "mount.h"
 #include "pixmaps.h"
@@ -57,12 +58,31 @@ struct _FileStatus
 	gchar	*text;	/* String so far */
 };
 
+typedef struct du {
+	gchar        *path;
+	GtkListStore *store;
+	guint         watch;
+	GIOChannel   *chan;
+	gint          child;
+} DU;
+
+typedef struct _Permissions Permissions;
+
+struct _Permissions
+{
+	gchar *path;
+	DirItem *item;
+	GtkWidget *bits[12];
+};
+
 /* Static prototypes */
 static void refresh_info(GObject *window);
-static GtkWidget *make_vbox(const guchar *path);
-static GtkWidget *make_details(const guchar *path, DirItem *item);
+static GtkWidget *make_vbox(const guchar *path, GObject *window);
+static GtkWidget *make_details(const guchar *path, DirItem *item,
+				GObject *window);
 static GtkWidget *make_about(const guchar *path, XMLwrapper *ai);
 static GtkWidget *make_file_says(const guchar *path);
+static GtkWidget *make_permissions(const gchar *path, DirItem *item);
 static void add_file_output(FileStatus *fs,
 			    gint source, GdkInputCondition condition);
 static const gchar *pretty_type(DirItem *file, const guchar *path);
@@ -119,11 +139,11 @@ void infobox_new(const gchar *pathname)
 
 	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
 
-	details = make_vbox(path);
+	owindow = G_OBJECT(window);
+	details = make_vbox(path, owindow);
 	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(window)->vbox),
 				    details);
 
-	owindow = G_OBJECT(window);
 	g_object_set_data(owindow, "details", details);
 	g_object_set_data_full(owindow, "path", path, g_free);
 
@@ -137,16 +157,6 @@ void infobox_new(const gchar *pathname)
  *			INTERNAL FUNCTIONS			*
  ****************************************************************/
 
-static void show_help_clicked(const gchar *path)
-{
-	DirItem *item;
-
-	item = diritem_new("");
-	diritem_restat(path, item, NULL);
-	show_item_help(path, item);
-	diritem_free(item);
-}
-
 static void got_response(GObject *window, gint response, gpointer data)
 {
 	if (response == GTK_RESPONSE_APPLY)
@@ -171,7 +181,7 @@ static void refresh_info(GObject *window)
 	vbox = details->parent;
 	gtk_widget_destroy(details);
 
-	details = make_vbox(path);
+	details = make_vbox(path, window);
 	g_object_set_data(window, "details", details);
 	gtk_box_pack_start_defaults(GTK_BOX(vbox), details);
 	gtk_widget_show_all(details);
@@ -190,14 +200,14 @@ static void add_frame(GtkBox *vbox, GtkWidget *list)
 /* Create the VBox widget that contains the details.
  * Note that 'path' must not be freed until the vbox is destroyed.
  */
-static GtkWidget *make_vbox(const guchar *path)
+static GtkWidget *make_vbox(const guchar *path, GObject *window)
 {
 	DirItem		*item;
 	GtkBox		*vbox;
 	XMLwrapper	*ai;
 	xmlNode 	*about = NULL;
 	gchar		*help_dir;
-	GtkWidget	*hbox, *name;
+	GtkWidget	*hbox, *name, *label;
 
 	g_return_val_if_fail(path[0] == '/', NULL);
 	
@@ -209,10 +219,11 @@ static GtkWidget *make_vbox(const guchar *path)
 		about = xml_get_section(ai, NULL, "About");
 
 	vbox = GTK_BOX(gtk_vbox_new(FALSE, 4));
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
 
 	/* Heading, with icon and name */
 	hbox = gtk_hbox_new(FALSE, 4);
-	gtk_container_add(GTK_CONTAINER(vbox), hbox);
+	gtk_box_pack_start(vbox, hbox, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox),
 			   gtk_image_new_from_pixbuf(item->image->pixbuf),
 			   FALSE, FALSE, 4);
@@ -251,7 +262,7 @@ static GtkWidget *make_vbox(const guchar *path)
 	}
 
 	/* List of file attributes */
-	add_frame(vbox, make_details(path, item));
+	add_frame(vbox, make_details(path, item, window));
 
 	help_dir = g_strconcat(path, "/Help", NULL);
 
@@ -266,15 +277,31 @@ static GtkWidget *make_vbox(const guchar *path)
 		gtk_box_pack_start(vbox, align, FALSE, TRUE, 0);
 		gtk_container_add(GTK_CONTAINER(align), button);
 		g_signal_connect_swapped(button, "clicked", 
-				G_CALLBACK(show_help_clicked),
+				G_CALLBACK(show_help_files),
 				(gpointer) path);
 	}
 	g_free(help_dir);
 
+	label = gtk_label_new(NULL);
+	gtk_label_set_markup(GTK_LABEL(label), _("<b>Permissions</b>"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 1);
+	gtk_box_pack_start(vbox, label, FALSE, TRUE, 2);
+
+	gtk_box_pack_start(vbox, make_permissions(path, item),
+			FALSE, TRUE, 0);
+
 	if (about)
 		add_frame(vbox, make_about(path, ai));
-	else
+	else if (item->base_type == TYPE_FILE)
+	{
+		label = gtk_label_new(NULL);
+		gtk_label_set_markup(GTK_LABEL(label),
+				_("<b>Contents indicate...</b>"));
+		gtk_misc_set_alignment(GTK_MISC(label), 0, 1);
+		gtk_box_pack_start(vbox, label, FALSE, TRUE, 2);
+
 		gtk_box_pack_start_defaults(vbox, make_file_says(path));
+	}
 
 	if (ai)
 		g_object_unref(ai);
@@ -312,10 +339,14 @@ static void set_selection(GtkTreeView *view, gpointer data)
 	g_free(text);
 }
 
-static void add_row(GtkListStore *store, const gchar *label, const gchar *data)
+/* Returns a GtkTreePath for the item */
+static const gchar *add_row(GtkListStore *store, const gchar *label,
+		      const gchar *data)
 {
 	GtkTreeIter	iter;
 	gchar		*u8 = NULL;
+	GtkTreePath     *tpath;
+	static gchar    *last = NULL;
 
 	if (!g_utf8_validate(data, -1, NULL))
 		u8 = to_utf8(data);
@@ -324,6 +355,14 @@ static void add_row(GtkListStore *store, const gchar *label, const gchar *data)
 	gtk_list_store_set(store, &iter, 0, label, 1, u8 ? u8 : data, -1);
 
 	g_free(u8);
+
+	tpath = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter);
+	if (last)
+		g_free(last);
+	last = gtk_tree_path_to_string(tpath);
+	gtk_tree_path_free(tpath);
+
+	return last;
 }
 
 static void add_row_and_free(GtkListStore *store,
@@ -334,13 +373,16 @@ static void add_row_and_free(GtkListStore *store,
 }
 
 /* Create an empty list view, ready to place some data in */
-static void make_list(GtkListStore **list_store, GtkWidget **list_view)
+static void make_list(GtkListStore **list_store, GtkWidget **list_view,
+			GCallback cell_edited)
 {
 	GtkListStore	*store;
 	GtkTreeView	*view;
 	GtkCellRenderer *cell_renderer;
 
-	store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	/* Field name, value, editable */
+	store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING,
+					G_TYPE_BOOLEAN);
 	view = GTK_TREE_VIEW(
 			gtk_tree_view_new_with_model(GTK_TREE_MODEL(store)));
 	g_object_unref(G_OBJECT(store));
@@ -353,7 +395,12 @@ static void make_list(GtkListStore **list_store, GtkWidget **list_view)
 
 	cell_renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_insert_column_with_attributes(view,
-			1, NULL, cell_renderer, "text", 1, NULL);
+			1, NULL, cell_renderer, "text", 1, "editable", 2, NULL);
+
+	if (cell_edited) {
+		g_signal_connect(G_OBJECT(cell_renderer), "edited",
+				G_CALLBACK(cell_edited), store);
+	}
 
 	g_signal_connect(view, "cursor_changed",
 			G_CALLBACK(set_selection), NULL);
@@ -361,17 +408,143 @@ static void make_list(GtkListStore **list_store, GtkWidget **list_view)
 	*list_store = store;
 	*list_view = (GtkWidget *) view;
 }
+
+static void set_cell(GtkListStore *store, const gchar *path,
+		     const gchar *ctext)
+{
+	GtkTreeIter iter;
+
+	gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(store),
+					    &iter, path);
+	gtk_list_store_set(store, &iter, 1, ctext, -1);
+}
+
+static void insert_size(DU *du, const char *line)
+{
+	off_t size;
+	gchar *cell;
+
+#ifdef	LARGE_FILE_SUPPORT
+	size = strtoll(line, NULL, 10);
+#else
+	size = strtol(line, NULL, 10);
+#endif
+	size <<= 10; /* Because du reports in K */
+	cell = (size >= PRETTY_SIZE_LIMIT)
+		? g_strdup_printf("%s (%" SIZE_FMT " %s)",
+				format_size(size),
+				size, _("bytes"))
+		: g_strdup(format_size(size));
+
+	set_cell(du->store, du->path, cell);
+
+	g_free(cell);
+}
+
+static gboolean read_du_output(GIOChannel *source, GIOCondition cond, DU *du)
+{
+	GString *line;
+	GIOStatus stat;
+	GError *err = NULL;
+
+	line = g_string_new("");
+	stat = g_io_channel_read_line_string(source, line, NULL, &err);
+	switch (stat)
+	{
+		case G_IO_STATUS_NORMAL:
+			insert_size(du, line->str);
+			break;
+		case G_IO_STATUS_EOF:
+			break;
+		case G_IO_STATUS_AGAIN:
+			g_string_free(line, TRUE);
+			return TRUE;
+		case G_IO_STATUS_ERROR:
+			set_cell(du->store, du->path, err->message);
+			break;
+	}
+	g_string_free(line, TRUE);
 	
+	return FALSE;
+}
+
+static void kill_du_output(GtkWidget *widget, DU *du)
+{
+	g_io_channel_shutdown(du->chan, FALSE, NULL);
+	g_io_channel_unref(du->chan);
+	kill((pid_t) du->child, SIGTERM);
+	g_object_unref(G_OBJECT(du->store));
+	g_free(du->path);
+	g_free(du);
+}
+
+static gboolean refresh_info_idle(gpointer data)
+{
+	GObject *window = G_OBJECT(data);
+
+	refresh_info(window);
+	g_object_unref(window);
+	return FALSE;
+}
+
+static void cell_edited(GtkCellRendererText *cell,
+	     const gchar *path_string,
+	     const gchar *new_text,
+	     gpointer data)
+{
+	GtkTreeModel *model = (GtkTreeModel *) data;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GObject *window;
+	const char *fullpath;
+	char *oldlink;
+
+	window = g_object_get_data(G_OBJECT(model), "rox_window");
+	g_return_if_fail(window != NULL);
+
+	fullpath = g_object_get_data(window, "path");
+	g_return_if_fail(fullpath != NULL);
+
+	path = gtk_tree_path_new_from_string(path_string);
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_path_free(path);
+
+	oldlink = readlink_dup(fullpath);
+	if (!oldlink) {
+		report_error(_("'%s' is no longer a symlink"), fullpath);
+		return;
+	}
+	if (strcmp(oldlink, new_text) == 0)
+		return;	/* No change */
+	g_free(oldlink);
+	if (unlink(fullpath)) {
+		report_error(_("Failed to unlink '%s':\n%s"),
+				fullpath, g_strerror(errno));
+		return;
+	}
+	if (symlink(new_text, fullpath)) {
+		report_error(_("Failed to create symlink from '%s':\n%s\n"
+				"(note: old link has been deleted)"));
+		return;
+	}
+
+	g_object_ref(window);
+	gtk_idle_add(refresh_info_idle, window);
+}
+
 /* Create the TreeView widget with the file's details */
-static GtkWidget *make_details(const guchar *path, DirItem *item)
+static GtkWidget *make_details(const guchar *path, DirItem *item,
+				GObject *window)
 {
 	GtkListStore	*store;
 	GtkWidget	*view;
 	gchar		*tmp, *tmp2;
 
-	make_list(&store, &view);
+	make_list(&store, &view, G_CALLBACK(cell_edited));
+	g_object_set_data(G_OBJECT(store), "rox_window", window);
 
-	if (item->base_type == TYPE_ERROR)
+	/* For a symlink to an error, don't show the error */
+	if (item->base_type == TYPE_ERROR && item->lstat_errno)
 	{
 		add_row(store, _("Error:"), g_strerror(item->lstat_errno));
 		return view;
@@ -389,12 +562,45 @@ static GtkWidget *make_details(const guchar *path, DirItem *item)
 					 group_name(item->gid)));
 
 	if (item->base_type != TYPE_DIRECTORY)
+	{
 		add_row_and_free(store, _("Size:"),
 				 item->size >= PRETTY_SIZE_LIMIT
 				 ? g_strdup_printf("%s (%" SIZE_FMT " %s)",
 						   format_size(item->size),
 						   item->size, _("bytes"))
 				 : g_strdup(format_size(item->size)));
+	}
+	else
+	{
+		DU *du;
+		int out;
+		gchar *args[] = {"du", "-sk", "", NULL};
+
+		du = g_new(DU, 1);
+		du->store = store;
+		du->path = g_strdup(add_row(store, _("Size:"), _("Scanning")));
+
+		args[2] = (gchar *) path;
+		if (g_spawn_async_with_pipes(NULL, args, NULL,
+					    G_SPAWN_SEARCH_PATH,
+					    NULL, NULL, &du->child,
+					    NULL, &out, NULL,
+					    NULL))
+		{
+			du->chan = g_io_channel_unix_new(out);
+			du->watch = g_io_add_watch(du->chan, G_IO_IN,
+						 (GIOFunc) read_du_output, du);
+			g_object_ref(G_OBJECT(du->store));
+ 			g_signal_connect(G_OBJECT(view), "destroy",
+ 				  G_CALLBACK(kill_du_output), du);
+		}
+		else
+		{
+			set_cell(store, du->path, _("Failed to scan"));
+			g_free(du->path);
+			g_free(du);
+		}
+	}
 
 	add_row_and_free(store, _("Change time:"), pretty_time(&item->ctime));
 	
@@ -402,13 +608,29 @@ static GtkWidget *make_details(const guchar *path, DirItem *item)
 
 	add_row_and_free(store, _("Access time:"), pretty_time(&item->atime));
 
-	add_row(store, _("Permissions:"), pretty_permissions(item->mode));
-
 	add_row(store, _("Type:"), pretty_type(item, path));
 
 	if (item->mime_type)
 		add_row(store, "", mime_type_comment(item->mime_type));
 
+	if (item->flags & ITEM_FLAG_SYMLINK)
+	{
+		GtkTreeIter iter;
+		GtkTreeModel *model = GTK_TREE_MODEL(store);
+		char *target;
+
+		target = readlink_dup(path);
+		if (!target)
+			target = g_strdup(g_strerror(errno));
+		add_row_and_free(store, _("Link target:"), target);
+
+		/* Make cell editable */
+		gtk_tree_model_iter_nth_child(model, &iter,
+			NULL, gtk_tree_model_iter_n_children(model, NULL) - 1);
+
+		gtk_list_store_set(store, &iter, 2, TRUE, -1);
+	}
+
 	if (item->base_type != TYPE_DIRECTORY)
 		add_row_and_free(store, _("Run action:"),
 				 describe_current_command(item->mime_type));
@@ -440,7 +662,7 @@ static GtkWidget *make_about(const guchar *path, XMLwrapper *ai)
 
 	g_return_val_if_fail(about != NULL, NULL);
 	
-	make_list(&store, &view);
+	make_list(&store, &view, NULL);
 
 	/* Add each field in about to the list, but overriding each element
 	 * with about_trans if a translation is supplied.
@@ -493,7 +715,6 @@ static GtkWidget *make_about(const guchar *path, XMLwrapper *ai)
 
 static GtkWidget *make_file_says(const guchar *path)
 {
-	GtkWidget	*frame;
 	GtkWidget	*w_file_label;
 	GtkLabel	*l_file_label;
 	int		file_data[2];
@@ -501,19 +722,16 @@ static GtkWidget *make_file_says(const guchar *path)
 	FileStatus 	*fs = NULL;
 	guchar 		*tmp;
 
-	frame = gtk_frame_new(_("file(1) says..."));
 	w_file_label = gtk_label_new(_("<nothing yet>"));
 	l_file_label = GTK_LABEL(w_file_label);
-	gtk_misc_set_padding(GTK_MISC(w_file_label), 4, 4);
 	gtk_label_set_line_wrap(l_file_label, TRUE);
-	gtk_container_add(GTK_CONTAINER(frame), w_file_label);
 	
 	if (pipe(file_data))
 	{
 		tmp = g_strdup_printf("pipe(): %s", g_strerror(errno));
 		gtk_label_set_text(l_file_label, tmp);
 		g_free(tmp);
-		return frame;
+		return w_file_label;
 	}
 
 	switch (fork())
@@ -550,12 +768,12 @@ static GtkWidget *make_file_says(const guchar *path)
 			fs->input = gtk_input_add_full(fs->fd, GDK_INPUT_READ,
 				(GdkInputFunction) add_file_output,
 				NULL, fs, NULL);
-			g_signal_connect(frame, "destroy",
+			g_signal_connect(w_file_label, "destroy",
 				G_CALLBACK(file_info_destroyed), fs);
 			break;
 	}
 
-	return frame;
+	return w_file_label;
 }
 
 /* Got some data from file(1) - stick it in the window. */
@@ -602,6 +820,109 @@ static void file_info_destroyed(GtkWidget *widget, FileStatus *fs)
 	g_free(fs);
 }
 
+static void permissions_destroyed(GtkWidget *widget, Permissions *perm)
+{
+	g_free(perm->path);
+	diritem_free(perm->item);
+	
+	g_free(perm);
+}
+
+static void permissions_apply(GtkWidget *widget, Permissions *perm)
+{
+	mode_t nmode;
+	int i;
+
+	nmode=0;
+	
+	for (i = 0; i < 9; i++)
+	{
+		GtkToggleButton *bit = GTK_TOGGLE_BUTTON(perm->bits[i]);
+		if (gtk_toggle_button_get_active(bit))
+			nmode |= 1 << i;
+	}
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(perm->bits[9])))
+		nmode |= S_ISUID;
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(perm->bits[10])))
+		nmode |= S_ISGID;
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(perm->bits[11])))
+		nmode |= S_ISVTX;
+	
+	if (chmod(perm->path, nmode))
+		report_error(_("Could not change permissions: %s"),
+			     g_strerror(errno));
+}
+
+static GtkWidget *make_permissions(const gchar *path, DirItem *item)
+{
+	Permissions *perm;
+	GtkWidget *table;
+	GtkWidget *tick, *label;
+	int i, x, y;
+
+	perm = g_new(Permissions, 1);
+
+	perm->path = g_strdup(path);
+	perm->item = diritem_new(path);
+
+	table = gtk_table_new(4, 5, TRUE);
+
+	label = gtk_label_new(_("Owner"));
+	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
+	label = gtk_label_new(_("Group"));
+	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
+	label = gtk_label_new(_("World"));
+	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
+
+	label = gtk_label_new(_("Read"));
+	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
+	label = gtk_label_new(_("Write"));
+	gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 0, 1);
+	label = gtk_label_new(_("Exec"));
+	gtk_table_attach_defaults(GTK_TABLE(table), label, 3, 4, 0, 1);
+
+	for (i = 0; i < 9; i++)
+	{
+		x = 1 + 2 - i % 3;
+		y = 1 + 2 - i / 3;
+		perm->bits[i] = tick = gtk_check_button_new();
+		gtk_table_attach_defaults(GTK_TABLE(table), tick,
+					  x, x + 1, y, y + 1);
+		if (item->mode & (1 << i))
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tick),
+						     TRUE);
+		g_signal_connect(tick, "toggled",
+				G_CALLBACK(permissions_apply), perm);
+	}
+
+	tick = gtk_check_button_new_with_label(_("SUID"));
+	gtk_table_attach_defaults(GTK_TABLE(table), tick, 4, 5, 1, 2);
+	if (item->mode & S_ISUID)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tick), TRUE);
+	g_signal_connect(tick, "toggled", G_CALLBACK(permissions_apply), perm);
+	perm->bits[9] = tick;
+
+	tick = gtk_check_button_new_with_label(_("SGID"));
+	gtk_table_attach_defaults(GTK_TABLE(table), tick, 4, 5, 2, 3);
+	if (item->mode & S_ISGID)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tick), TRUE);
+	g_signal_connect(tick, "toggled", G_CALLBACK(permissions_apply), perm);
+	perm->bits[10] = tick;
+
+	tick = gtk_check_button_new_with_label(_("Sticky"));
+	gtk_table_attach_defaults(GTK_TABLE(table), tick, 4, 5, 3, 4);
+	if (item->mode & S_ISVTX)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tick), TRUE);
+	g_signal_connect(tick, "toggled", G_CALLBACK(permissions_apply), perm);
+	perm->bits[11] = tick;
+
+	g_signal_connect(table, "destroy",
+				G_CALLBACK(permissions_destroyed), perm);
+			
+	gtk_widget_show_all(table);
+	return table;
+}
+
 /* Don't g_free() the result */
 static const gchar *pretty_type(DirItem *file, const guchar *path)
 {
@@ -610,22 +931,7 @@ static const gchar *pretty_type(DirItem *file, const guchar *path)
 	null_g_free(&text);
 
 	if (file->flags & ITEM_FLAG_SYMLINK)
-	{
-		char	*target;
-
-		target = readlink_dup(path);
-		if (target)
-		{
-			ensure_utf8(&target);
-
-			text = g_strdup_printf(_("Symbolic link to %s"),
-					       target);
-			g_free(target);
-			return text;
-		}
-
 		return _("Symbolic link");
-	}
 
 	if (file->flags & ITEM_FLAG_APPDIR)
 		return _("ROX application");
