@@ -1,5 +1,5 @@
 /*
- * $Id: menu.c,v 1.251.2.2 2003/07/23 16:03:07 tal197 Exp $
+ * $Id: menu.c,v 1.273 2004/01/21 20:03:15 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -66,8 +66,7 @@ typedef enum {
 	FILE_RENAME_ITEM,
 	FILE_LINK_ITEM,
 	FILE_OPEN_FILE,
-	FILE_HELP,
-	FILE_SHOW_FILE_INFO,
+	FILE_PROPERTIES,
 	FILE_RUN_ACTION,
 	FILE_SET_ICON,
 	FILE_SEND_TO,
@@ -76,15 +75,9 @@ typedef enum {
 	FILE_CHMOD_ITEMS,
 	FILE_FIND,
 	FILE_OPEN_VFS_AVFS,
+	FILE_SET_TYPE,
 } FileOp;
 
-typedef enum menu_icon_style {
-  MIS_NONE, MIS_SMALL, MIS_LARGE,
-  MIS_HUGE_UNUSED,
-  MIS_CURRENT, /* As per current filer window */
-  MIS_DEFAULT
-} MenuIconStyle;
-
 typedef void (*ActionFn)(GList *paths,
 			 const char *dest_dir, const char *leaf, int quiet);
 typedef void MenuCallback(GtkWidget *widget, gpointer data);
@@ -208,15 +201,15 @@ static GtkItemFactoryEntry filer_menu_def[] = {
 {">" N_("Link..."),		NULL, file_op, FILE_LINK_ITEM, NULL},
 {">" N_("Delete"),	    	NULL, file_op, FILE_DELETE, "<StockItem>", GTK_STOCK_DELETE},
 {">",				NULL, NULL, 0, "<Separator>"},
-{">" N_("Help"),		NULL, file_op, FILE_HELP, "<StockItem>", GTK_STOCK_HELP},
 {">" N_("Shift Open"),   	NULL, file_op, FILE_OPEN_FILE},
 {">" N_("Open AVFS"),		NULL, file_op, FILE_OPEN_VFS_AVFS, "<StockItem>", GTK_STOCK_OPEN},
 {">" N_("Send To..."),		NULL, file_op, FILE_SEND_TO, NULL},
 {">",				NULL, NULL, 0, "<Separator>"},
 {">" N_("Set Run Action..."),	NULL, file_op, FILE_RUN_ACTION, "<StockItem>", GTK_STOCK_EXECUTE},
 {">" N_("Set Icon..."),		NULL, file_op, FILE_SET_ICON, NULL},
-{">" N_("Info"),		NULL, file_op, FILE_SHOW_FILE_INFO, "<StockItem>", GTK_STOCK_DIALOG_INFO},
+{">" N_("Properties"),		NULL, file_op, FILE_PROPERTIES, "<StockItem>", GTK_STOCK_PROPERTIES},
 {">" N_("Count"),		NULL, file_op, FILE_USAGE, NULL},
+{">" N_("Set Type..."),		NULL, file_op, FILE_SET_TYPE, NULL},
 {">" N_("Permissions"),		NULL, file_op, FILE_CHMOD_ITEMS, NULL},
 {">",				NULL, NULL, 0, "<Separator>"},
 {">" N_("Find"),		NULL, file_op, FILE_FIND, "<StockItem>", GTK_STOCK_FIND},
@@ -304,7 +297,7 @@ gboolean ensure_filer_menu(void)
 
 	/* Shift Open... label */
 	items = gtk_container_get_children(GTK_CONTAINER(filer_file_menu));
-	file_shift_item = GTK_BIN(g_list_nth(items, 6)->data)->child;
+	file_shift_item = GTK_BIN(g_list_nth(items, 5)->data)->child;
 	g_list_free(items);
 
 	GET_SSMENU_ITEM(item, "filer", "Window", "New Window");
@@ -380,8 +373,8 @@ static void menuitem_no_shortcuts(GtkWidget *item)
 static void shade_file_menu_items(gboolean shaded)
 {
 	menu_set_items_shaded(filer_file_menu, shaded, 0, 3);
-	menu_set_items_shaded(filer_file_menu, shaded, 5, 3);
-	menu_set_items_shaded(filer_file_menu, shaded, 10, 2);
+	menu_set_items_shaded(filer_file_menu, shaded, 5, 2);
+	menu_set_items_shaded(filer_file_menu, shaded, 9, 2);
 }
 
 /* 'data' is an array of three ints:
@@ -424,44 +417,39 @@ void position_menu(GtkMenu *menu, gint *x, gint *y,
 	*push_in = FALSE;
 }
 
-/* Returns an array listing all the names in the directory 'path'.
- * The array is unsorted.
- * '.' and '..' are skipped.
- * On error, the error is reported with g_warning and NULL is returned.
- */
-static GPtrArray *list_dir(const guchar *path)
+GtkWidget *make_send_to_item(DirItem *ditem, const char *label,
+				MenuIconStyle style)
 {
-	GDir *dir;
-	GError *error = NULL;
-	GPtrArray *names;
-	const char *leaf;
+	GtkWidget *item;
 	
-	dir = g_dir_open(path, 0, &error);
-	if (error)
+	if (ditem->image && style != MIS_NONE)
 	{
-		g_warning("Can't list directory:\n%s", error->message);
-		g_error_free(error);
-		return NULL;
+		GdkPixbuf *pixbuf;
+
+		switch (style)
+		{
+			case MIS_LARGE:
+				pixbuf = ditem->image->pixbuf;
+				break;
+			default:
+				if (!ditem->image->sm_pixbuf)
+					pixmap_make_small(ditem->image);
+				pixbuf = ditem->image->sm_pixbuf;
+				break;
+		}
+
+		item = gtk_image_menu_item_new_with_label(label);
+		/* TODO: Find a way to allow short-cuts */
+		menuitem_no_shortcuts(item);
+
+		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
+				gtk_image_new_from_pixbuf(pixbuf));
+		gtk_widget_show_all(item);
 	}
+	else
+		item = gtk_menu_item_new_with_label(label);
 
-	names = g_ptr_array_new();
-
-	while ((leaf = g_dir_read_name(dir)))
-		if (leaf[0] != '.')
-			g_ptr_array_add(names, g_strdup(leaf));
-
-	g_dir_close(dir);
-
-	return names;
-}
-
-/* Used as the sort function for sorting GPtrArrays */
-static gint strcmp2(gconstpointer a, gconstpointer b)
-{
-	const char *aa = *(char **) a;
-	const char *bb = *(char **) b;
-
-	return g_strcasecmp(aa, bb);
+	return item;
 }
 
 static GList *menu_from_dir(GtkWidget *menu, const gchar *dir_name,
@@ -489,8 +477,6 @@ static GList *menu_from_dir(GtkWidget *menu, const gchar *dir_name,
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	}
 
-	g_ptr_array_sort(names, strcmp2);
-
 	for (i = 0; i < names->len; i++)
 	{
 		char	*leaf = names->pdata[i];
@@ -510,31 +496,7 @@ static GList *menu_from_dir(GtkWidget *menu, const gchar *dir_name,
 		ditem = diritem_new("");
 		diritem_restat(fname, ditem, NULL);
 
-		if (ditem->image && style != MIS_NONE)
-		{
-			GdkPixbuf *pixbuf;
-
-			switch (style)
-			{
-				case MIS_LARGE:
-					pixbuf = ditem->image->pixbuf;
-					break;
-				default:
-					if (!ditem->image->sm_pixbuf)
-						pixmap_make_small(ditem->image);
-					pixbuf = ditem->image->sm_pixbuf;
-					break;
-			}
-
-			item = gtk_image_menu_item_new_with_label(leaf);
-			/* TODO: Find a way to allow short-cuts */
-			menuitem_no_shortcuts(item);
-
-			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-					gtk_image_new_from_pixbuf(pixbuf));
-		}
-		else
-			item = gtk_menu_item_new_with_label(leaf);
+		item = make_send_to_item(ditem, leaf, style);
 
 		g_free(leaf);
 
@@ -694,6 +656,7 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 	DirItem		*file_item = NULL;
 	GdkModifierType	state = 0;
 	int		n_selected;
+	int             n_added = 0;
 
 	n_selected = view_count_selected(filer_window->view);
 
@@ -781,7 +744,7 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 							item->leafname);
 				shade_file_menu_items(FALSE);
 				file_item = filer_selected_item(filer_window);
-				g_string_printf(buffer, "%s '%s'",
+				g_string_printf(buffer, _("%s '%s'"),
 					basetype_name(file_item),
 					g_utf8_validate(file_item->leafname,
 							-1, NULL)
@@ -789,7 +752,7 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 						: _("(bad utf-8)"));
 				if (!can_set_run_action(file_item))
 					menu_set_items_shaded(filer_file_menu,
-							TRUE, 6, 1);
+							TRUE, 9, 1);
 				break;
 			default:
 				shade_file_menu_items(TRUE);
@@ -803,9 +766,9 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 		menu_show_shift_action(file_shift_item, file_item,
 					n_selected == 0);
 		if (file_item)
-			appmenu_add(make_path(filer_window->sym_path,
-						file_item->leafname),
-					file_item, filer_file_menu);
+			n_added = appmenu_add(make_path(filer_window->sym_path,
+							file_item->leafname),
+						file_item, filer_file_menu);
 	}
 
 	update_new_files_menu(get_menu_icon_style());
@@ -822,7 +785,7 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 	updating_menu--;
 	
 	show_popup_menu(popup_menu, event,
-			popup_menu == filer_file_menu ? 5 : 1);
+			popup_menu == filer_file_menu ? (n_added + 7) : 1);
 }
 
 static void menu_closed(GtkWidget *widget)
@@ -837,6 +800,8 @@ static void menu_closed(GtkWidget *widget)
 		view_clear_selection(window_with_focus->view);
 		window_with_focus->temp_item_selected = FALSE;
 	}
+
+	appmenu_remove();
 }
 
 static void target_callback(FilerWindow *filer_window,
@@ -991,7 +956,7 @@ static void refresh(gpointer data, guint action, GtkWidget *widget)
 {
 	g_return_if_fail(window_with_focus != NULL);
 
-	full_refresh();
+	filer_refresh(window_with_focus);
 }
 
 static void delete(FilerWindow *filer_window)
@@ -1018,6 +983,14 @@ static void chmod_items(FilerWindow *filer_window)
 	destroy_glist(&paths);
 }
 
+static void set_type_items(FilerWindow *filer_window)
+{
+	GList *paths;
+	paths = filer_selected_items(filer_window);
+	action_settype(paths, FALSE, NULL);
+	destroy_glist(&paths);
+}
+
 static void find(FilerWindow *filer_window)
 {
 	GList *paths;
@@ -1776,11 +1749,11 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 			case FILE_OPEN_FILE:
 				prompt = _("Shift Open ... ?");
 				break;
-			case FILE_HELP:
-				prompt = _("Help about ... ?");
+			case FILE_PROPERTIES:
+				prompt = _("Properties of ... ?");
 				break;
-			case FILE_SHOW_FILE_INFO:
-				prompt = _("Examine ... ?");
+			case FILE_SET_TYPE:
+				prompt = _("Set type of ... ?");
 				break;
 			case FILE_RUN_ACTION:
 				prompt = _("Set run action for ... ?");
@@ -1829,10 +1802,13 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 		case FILE_CHMOD_ITEMS:
 			chmod_items(window_with_focus);
 			return;
+		case FILE_SET_TYPE:
+			set_type_items(window_with_focus);
+			return;
 		case FILE_FIND:
 			find(window_with_focus);
 			return;
-		case FILE_SHOW_FILE_INFO:
+		case FILE_PROPERTIES:
 		{
 			GList *items;
 
@@ -1893,9 +1869,6 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 			filer_openitem(window_with_focus, &iter,
 				OPEN_SAME_WINDOW | OPEN_SHIFT);
 			break;
-		case FILE_HELP:
-			show_item_help(path, item);
-			break;
 		case FILE_RUN_ACTION:
 			run_action(item);
 			break;
@@ -1923,11 +1896,12 @@ static void show_key_help(GtkWidget *button, gpointer data)
 	{
 		info_message(_("User-definable shortcuts are disabled by "
 			"default in Gtk2, and you have not enabled "
-			"them. You can turn this feature on by:\n"
-			"1) using an XSettings manager, such as ROX-Session\n"
-			"or\n"
+			"them. You can turn this feature on by:\n\n"
+			"1) using an XSettings manager, such as ROX-Session "
+			"or gnome-settings-daemon, or\n\n"
 			"2) adding this line to ~/.gtkrc-2.0:\n"
-			"\tgtk-can-change-accels = 1"));
+			"\tgtk-can-change-accels = 1\n"
+			"\t(this only works if NOT using XSETTINGS)"));
 		return;
 	}
 
