@@ -1,5 +1,5 @@
 /*
- * $Id: minibuffer.c,v 1.63 2003/01/19 19:29:04 tal197 Exp $
+ * $Id: minibuffer.c,v 1.65 2003/12/26 22:40:22 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -420,8 +420,11 @@ static void path_changed(FilerWindow *filer_window)
 		/* The new path is in a different directory */
 		struct stat info;
 
-		if (mc_stat(path, &info) == 0 && S_ISDIR(info.st_mode))
+		if (stat_with_timeout(path, &info) == 0 &&
+				S_ISDIR(info.st_mode))
+		{
 			filer_change_to(filer_window, path, leaf);
+		}
 		else
 			error = TRUE;
 	}
@@ -952,7 +955,7 @@ static const gchar *mini_contents(FilerWindow *filer_window)
 	entry = gtk_entry_get_text(GTK_ENTRY(filer_window->minibuffer));
 
 	for (c = entry; *c; c++)
-		if (!isspace(*c))
+		if (!g_ascii_isspace(*c))
 			return entry;
 
 	return NULL;
