@@ -1,5 +1,5 @@
 /*
- * $Id: panel.c,v 1.104.2.1 2003/07/23 15:45:10 tal197 Exp $
+ * $Id: panel.c,v 1.109 2003/11/29 17:46:43 kerofin Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -1107,12 +1107,14 @@ static void add_uri_list(GtkWidget          *widget,
 
 	for (next = uris; next; next = next->next)
 	{
-		const guchar	*path;
+		guchar	*path;
 
 		path = get_local_path((guchar *) next->data);
 
-		if (path)
+		if (path) {
 			panel_add_item(panel, path, NULL, after, NULL);
+			g_free(path);
+		}
 	}
 
 	g_list_free(uris);
@@ -1264,7 +1266,10 @@ static gint panel_leave_event(GtkWidget *widget,
 			      Panel *panel)
 {
 	GdkWindow *pinboard;
-	
+
+	if (event->mode != GDK_CROSSING_NORMAL)
+		return FALSE;	/* Grab for menu, DnD, etc */
+
 	pinboard = pinboard_get_window();
 	window_put_just_above(panel->window->window, pinboard);
 
@@ -1923,6 +1928,9 @@ static gboolean panel_want_show_text(PanelIcon *pi)
 {
 	Icon *icon = (Icon *) pi;
 
+	if (!icon->item->leafname[0])
+		return FALSE;
+
 	if (o_panel_style.int_value == SHOW_BOTH)
 		return TRUE;
 	if (o_panel_style.int_value == SHOW_ICON)
