@@ -25,10 +25,11 @@ void pinboard_pin(const gchar *path, const gchar *name, int x, int y,
 		  const gchar *shortcut);
 void pinboard_move_icons(void);
 const gchar *pinboard_get_name(void);
-void pinboard_set_backdrop(void);
+void pinboard_set_backdrop_box(void);
 void pinboard_set_backdrop_app(const gchar *app);
 GdkWindow *pinboard_get_window(void);
 void pinboard_add_widget(GtkWidget *widget);
 void draw_label_shadow(WrappedLabel *wl, GdkRegion *region);
+void pinboard_set_backdrop(const gchar *path, BackdropStyle style);
 
 #endif /* _PINBOARD_H */
