@@ -1,5 +1,5 @@
 /*
- * $Id: pixmaps.c,v 1.103 2003/05/27 13:03:16 tal197 Exp $
+ * $Id: pixmaps.c,v 1.113 2004/01/21 20:03:16 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -29,6 +29,7 @@
  */
 
 #define PIXMAP_PURGE_TIME 1200
+#define PIXMAP_THUMB_SIZE  128
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -36,6 +37,7 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <string.h>
 
 #include <gtk/gtk.h>
 
@@ -48,8 +50,11 @@
 #include "main.h"
 #include "filer.h"
 #include "dir.h"
+#include "diritem.h"
+#include "choices.h"
 #include "options.h"
 #include "action.h"
+#include "type.h"
 
 GFSCache *pixmap_cache = NULL;
 
@@ -93,6 +98,7 @@ struct _ChildThumbnail {
 static const char *stocks[] = {
 	ROX_STOCK_SHOW_DETAILS,
 	ROX_STOCK_SHOW_HIDDEN,
+	ROX_STOCK_SELECT,
 	ROX_STOCK_MOUNT,
 	ROX_STOCK_MOUNTED,
 };
@@ -112,6 +118,8 @@ static void child_create_thumbnail(const gchar *path);
 static GdkPixbuf *create_spotlight_pixbuf(GdkPixbuf *src, guint32 color,
 					  guchar alpha);
 static GList *thumbs_purge_cache(Option *option, xmlNode *node, guchar *label);
+static gchar *thumbnail_path(const gchar *path);
+static gchar *thumbnail_program(MIME_type *type);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -264,6 +272,9 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 	GdkPixbuf	*pixbuf;
 	pid_t		child;
 	ChildThumbnail	*info;
+	MIME_type       *type;
+	gchar		*thumb_prog;
+
 
 	image = g_fscache_lookup_full(pixmap_cache, path,
 					FSCACHE_LOOKUP_ONLY_NEW, &found);
@@ -271,7 +282,8 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 	if (found)
 	{
 		/* Thumbnail is known, or being created */
-		g_object_unref(image);
+		if (image)
+			g_object_unref(image);
 		callback(data, NULL);
 		return;
 	}
@@ -325,15 +337,29 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 		return;
 	}
 
+	type = type_from_path(path);
+	if (!type)
+		type = text_plain;
+
 	/* Add an entry, set to NULL, so no-one else tries to load this
 	 * image.
 	 */
 	g_fscache_insert(pixmap_cache, path, NULL, TRUE);
 
+	thumb_prog = thumbnail_program(type);
+
+	/* Only attempt to load 'images' types ourselves */
+	if (thumb_prog == NULL && strcmp(type->media_type, "image") != 0)
+	{
+		callback(data, NULL);
+		return;		/* Don't know how to handle this type */
+	}
+
 	child = fork();
 
 	if (child == -1)
 	{
+		g_free(thumb_prog);
 		delayed_error("fork(): %s", g_strerror(errno));
 		callback(data, NULL);
 		return;
@@ -341,11 +367,32 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 
 	if (child == 0)
 	{
-		/* We are the child process */
+		/* We are the child process.  (We are sloppy with freeing
+		 memory, but since we go away very quickly, that's ok.) */
+		if (thumb_prog)
+		{
+			DirItem *item;
+			
+			item = diritem_new(g_basename(thumb_prog));
+
+			diritem_restat(thumb_prog, item, NULL);
+			if (item->flags & ITEM_FLAG_APPDIR)
+				thumb_prog = g_strconcat(thumb_prog, "/AppRun",
+						       NULL);
+						  
+			execl(thumb_prog, thumb_prog, path,
+			      thumbnail_path(path),
+			      g_strdup_printf("%d", PIXMAP_THUMB_SIZE),
+			      (void *)NULL);
+			_exit(1);
+		}
+
 		child_create_thumbnail(path);
 		_exit(0);
 	}
 
+	g_free(thumb_prog);
+
 	info = g_new(ChildThumbnail, 1);
 	info->path = g_strdup(path);
 	info->callback = callback;
@@ -369,7 +416,7 @@ static void save_thumbnail(const char *pathname, GdkPixbuf *full)
 	int name_len;
 	GdkPixbuf *thumb;
 
-	thumb = scale_pixbuf(full, 128, 128);
+	thumb = scale_pixbuf(full, PIXMAP_THUMB_SIZE, PIXMAP_THUMB_SIZE);
 
 	original_width = gdk_pixbuf_get_width(full);
 	original_height = gdk_pixbuf_get_height(full);
@@ -431,6 +478,57 @@ static void save_thumbnail(const char *pathname, GdkPixbuf *full)
 	g_free(uri);
 }
 
+static gchar *thumbnail_path(const char *path)
+{
+	gchar *uri, *md5;
+	GString *to;
+	gchar *ans;
+	
+	uri = g_strconcat("file://", path, NULL);
+	md5 = md5_hash(uri);
+		
+	to = g_string_new(home_dir);
+	g_string_append(to, "/.thumbnails");
+	mkdir(to->str, 0700);
+	g_string_append(to, "/normal/");
+	mkdir(to->str, 0700);
+	g_string_append(to, md5);
+	g_string_append(to, ".png");
+
+	g_free(md5);
+	g_free(uri);
+
+	ans=to->str;
+	g_string_free(to, FALSE);
+
+	return ans;
+}
+
+/* Return a program to create thumbnails for files of this type.
+ * NULL to try to make it ourself (using gdk).
+ * g_free the result.
+ */
+static gchar *thumbnail_program(MIME_type *type)
+{
+	gchar *leaf;
+	gchar *path;
+
+	if (!type)
+		return NULL;
+
+	leaf = g_strconcat(type->media_type, "_", type->subtype, NULL);
+	path = choices_find_path_load(leaf, "MIME-thumb");
+	if (path)
+	{
+		g_free(leaf);
+		return path;
+	}
+
+	path = choices_find_path_load(type->media_type, "MIME-thumb");
+
+	return path;
+}
+
 /* Called in a subprocess. Load path and create the thumbnail
  * file. Parent will notice when we die.
  */
@@ -506,7 +604,7 @@ static GdkPixbuf *get_thumbnail_for(const char *pathname)
 	
 	if (mc_stat(path, &info) != 0)
 		goto err;
-	
+
 	if (info.st_mtime != atol(smtime) || info.st_size != atol(ssize))
 		goto err;
 
