@@ -1,5 +1,5 @@
 /*
- * $Id: remote.c,v 1.46 2003/05/29 10:33:42 tal197 Exp $
+ * $Id: remote.c,v 1.52 2003/10/26 16:14:28 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -96,6 +96,7 @@ static xmlNodePtr rpc_Mount(GList *args);
 
 static xmlNodePtr rpc_PanelAdd(GList *args);
 static xmlNodePtr rpc_PinboardAdd(GList *args);
+static xmlNodePtr rpc_SetBackdrop(GList *args);
 static xmlNodePtr rpc_SetBackdropApp(GList *args);
 
 /****************************************************************
@@ -122,7 +123,7 @@ gboolean remote_init(xmlDocPtr rpc, gboolean new_copy)
 
 	soap_register("Run", rpc_Run, "Filename", NULL);
 	soap_register("OpenDir", rpc_OpenDir, "Filename",
-					      "Style,Details,Sort,Class");
+					      "Style,Details,Sort,Class,ID");
 	soap_register("CloseDir", rpc_CloseDir, "Filename", NULL);
 	soap_register("Examine", rpc_Examine, "Filename", NULL);
 	soap_register("Show", rpc_Show, "Directory,Leafname", NULL);
@@ -137,6 +138,7 @@ gboolean remote_init(xmlDocPtr rpc, gboolean new_copy)
 	soap_register("Link", rpc_Link, "From,To", "Leafname");
 	soap_register("Mount", rpc_Mount, "MountPoints", "OpenDir,Quiet");
 
+	soap_register("SetBackdrop", rpc_SetBackdrop, "Filename,Style", NULL);
 	soap_register("SetBackdropApp", rpc_SetBackdropApp, "App", NULL);
 	soap_register("PinboardAdd", rpc_PinboardAdd, "Path,X,Y", "Label");
 	soap_register("PanelAdd", rpc_PanelAdd, "Side,Path", "Label,After");
@@ -424,7 +426,12 @@ static gboolean client_event(GtkWidget *window,
 		return FALSE;
 
 	src_window = gdk_window_foreign_new(event->data.l[0]);
-	g_return_val_if_fail(src_window != NULL, FALSE);
+	if (!src_window)
+	{
+		g_warning("SOAP message sender window was destroyed before I \n"
+			  "could read it.");
+		return FALSE;
+	}
 	prop = gdk_x11_xatom_to_atom(event->data.l[1]);
 
 	data = read_property(src_window, prop, &length);
@@ -556,23 +563,37 @@ static xmlNodePtr rpc_Version(GList *args)
 
 	reply = xmlNewNode(NULL, "rox:VersionResponse");
 	xmlNewNs(reply, SOAP_RPC_NS, "soap");
-	xmlNewChild(reply, NULL, "soap:result", VERSION);
+	xmlNewTextChild(reply, NULL, "soap:result", VERSION);
 
 	return reply;
 }
 
-/* Args: Path, [Style, Details, Sort, Class] */
+/* Args: Path, [Style, Details, Sort, Class, Window] */
 static xmlNodePtr rpc_OpenDir(GList *args)
 {
 	char	   *path;
-	char       *style, *details, *sort, *class;
-	FilerWindow *fwin;
+	char       *style, *details, *sort, *class, *window;
+	FilerWindow *fwin = NULL;
 
 	path = string_value(ARG(0));
 	class = string_value(ARG(4));
-	fwin = filer_opendir(path, NULL, class);
+	window = string_value(ARG(5));
+	
+	if (window)
+		fwin = filer_get_by_id(window);
+
+	if (!fwin)
+	{
+		fwin = filer_opendir(path, NULL, class);
+		if (window)
+			filer_set_id(fwin, window); 
+	}
+	else
+		filer_change_to(fwin, path, NULL);
+
 	g_free(path);
 	g_free(class);
+	g_free(window);
 	if (!fwin)
 		return NULL;
 
@@ -710,6 +731,33 @@ static xmlNodePtr rpc_Pinboard(GList *args)
 	return NULL;
 }
 
+/* args = Filename, Style */
+static xmlNodePtr rpc_SetBackdrop(GList *args)
+{
+	char *file;
+	char *style;
+	BackdropStyle s;
+
+	file = string_value(ARG(0));
+	style = string_value(ARG(1));
+
+	s = !g_strcasecmp(style, "Tile") ? BACKDROP_TILE :
+	    !g_strcasecmp(style, "Scale") ? BACKDROP_SCALE :
+	    !g_strcasecmp(style, "Stretch") ? BACKDROP_STRETCH :
+	    !g_strcasecmp(style, "Centre") ? BACKDROP_CENTRE :
+					     BACKDROP_NONE;
+
+	if (s == BACKDROP_NONE)
+		g_warning("Invalid style '%s' for backdrop", style);
+	else
+		pinboard_set_backdrop(file, s);
+
+	g_free(file);
+	g_free(style);
+
+	return NULL;
+}
+
 /* args = App */
 static xmlNodePtr rpc_SetBackdropApp(GList *args)
 {
@@ -889,7 +937,7 @@ static xmlNodePtr rpc_FileType(GList *args)
 	tname = g_strconcat(type->media_type, "/", type->subtype, NULL);
 
 	xmlNewNs(reply, SOAP_RPC_NS, "soap");
-	xmlNewChild(reply, NULL, "soap:result", tname);
+	xmlNewTextChild(reply, NULL, "soap:result", tname);
 	g_free(tname);
 
 	return reply;
@@ -997,9 +1045,9 @@ static xmlNodePtr soap_invoke(xmlNode *method)
 		reply = xmlNewNode(NULL, "env:Fault");
 		xmlNewNs(reply, SOAP_RPC_NS, "rpc");
 		xmlNewNs(reply, SOAP_ENV_NS, "env");
-		xmlNewChild(reply, NULL, "faultcode",
+		xmlNewTextChild(reply, NULL, "faultcode",
 						"rpc:ProcedureNotPresent");
-		xmlNewChild(reply, NULL, "faultstring", err);
+		xmlNewTextChild(reply, NULL, "faultstring", err);
 		g_free(err);
 		return reply;
 	}
