@@ -1,5 +1,5 @@
 /*
- * $Id: support.c,v 1.100 2003/05/29 11:04:51 tal197 Exp $
+ * $Id: support.c,v 1.110 2004/01/21 20:03:16 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -466,34 +466,36 @@ static gboolean is_local_address(char *address)
 }
 
 /* Convert a URI to a local pathname (or NULL if it isn't local).
- * The returned pointer points inside the input string.
+ * The returned pointer needs to be passed to g_free() when done (if not NULL).
+ * THIS IS A CHANGE. The return path has been processed by unescape_uri().
  * Possible formats:
  *	/path
  *	///path
  *	//host/path
  *	file://host/path
  */
-const char *get_local_path(const char *uri)
+char *get_local_path(const char *uri)
 {
 	if (*uri == '/')
 	{
 		char    *path, *uri_host;
 
+		/* Just a local path - no host part */
 		if (uri[1] != '/')
-			return uri;	/* Just a local path - no host part */
-
+			return unescape_uri(uri);
+		                    
 		path = strchr(uri + 2, '/');
 		if (!path)
 			return NULL;	    /* //something */
 
 		if (path - uri == 2)
-			return path;	/* ///path */
+			return unescape_uri(path);	/* ///path */
 		
 		uri_host = g_strndup(uri + 2, path - uri - 2);
 		if (is_local_address(uri_host))
 		{
 			g_free(uri_host);
-			return path;	/* //myhost/path */
+			return unescape_uri(path);	/* //myhost/path */
 		}
 		g_free(uri_host);
 
@@ -655,12 +657,13 @@ gboolean in_list(const guchar *item, const guchar *list)
 	
 	while (*list)
 	{
-		if (strncmp(item, list, len) == 0 && !isalpha(list[len]))
+		if (strncmp(item, list, len) == 0 &&
+		    !g_ascii_isalpha(list[len]))
 			return TRUE;
 		list = strchr(list, ',');
 		if (!list)
 			return FALSE;
-		while (isspace(*++list))
+		while (g_ascii_isspace(*++list))
 			;
 	}
 
@@ -782,7 +785,7 @@ int text_to_boolean(const char *text, int defvalue)
 	        return TRUE;
 	else if (g_strcasecmp(text, "no")==0)
 	        return FALSE;
-	else if (isdigit(text[0]))
+	else if (g_ascii_isdigit(text[0]))
 	        return !!atoi(text);
 
 	return defvalue;
@@ -1059,6 +1062,9 @@ gchar *to_utf8(const gchar *src)
 		return NULL;
 
 	retval = g_locale_to_utf8(src, -1, NULL, NULL, NULL);
+	if (!retval)
+		retval = g_convert_with_fallback(src, -1, "utf-8", "iso-8859-1",
+						"?", NULL, NULL, NULL);
 
 	return retval ? retval : g_strdup(src);
 }
@@ -1257,3 +1263,141 @@ gboolean file_exists(const char *path)
 
 	return !mc_stat(path, &info);
 }
+
+/* Escape path for future use in URI */
+gchar *escape_uri_path(const char *path)
+{
+	const char *safe = "-_./"; /* Plus isalnum() */
+	const guchar *s;
+	gchar *ans;
+	GString *str;
+
+	str = g_string_sized_new(strlen(path));
+
+	for (s = path; *s; s++)
+	{
+		if (!g_ascii_isalnum(*s) && !strchr(safe, *s))
+			g_string_append_printf(str, "%%%02x", *s);
+		else
+			str = g_string_append_c(str, *s);
+	}
+
+	ans = str->str;
+	g_string_free(str, FALSE);
+	
+	return ans;
+}
+
+gchar *encode_path_as_uri(const guchar *path)
+{
+	gchar *tpath = escape_uri_path(path);
+	gchar *uri;
+
+	uri = g_strconcat("file://", our_host_name_for_dnd(), tpath, NULL);
+	g_free(tpath);
+
+	return uri;
+}
+
+gchar *unescape_uri(const char *uri)
+{
+	const gchar *s;
+	gchar *d;
+	gchar *tmp;
+	
+	tmp = g_malloc(strlen(uri) + 1);
+	for (s = uri, d = tmp; *s; s++, d++)
+	{
+		/*printf("%s\n", s);*/
+		if (*s == '%' && g_ascii_isxdigit(s[1]) &&
+				 g_ascii_isxdigit(s[2]))
+		{
+			int c;
+			char buf[3];
+			buf[0] = s[1];
+			buf[1] = s[2];
+			buf[2] = 0;
+			c = (int) strtol(buf, NULL, 16);
+			*d = c;
+			s += 2;
+		}
+		else
+			*d = *s;
+	}
+	*d = '\0';
+
+	return tmp;
+}
+
+/* Used as the sort function for sorting GPtrArrays */
+gint strcmp2(gconstpointer a, gconstpointer b)
+{
+	const char *aa = *(char **) a;
+	const char *bb = *(char **) b;
+
+	return g_strcasecmp(aa, bb);
+}
+
+/* Returns an array listing all the names in the directory 'path'.
+ * The array is sorted.
+ * '.' and '..' are skipped.
+ * On error, the error is reported with g_warning and NULL is returned.
+ */
+GPtrArray *list_dir(const guchar *path)
+{
+	GDir *dir;
+	GError *error = NULL;
+	const char *leaf;
+	GPtrArray *names;
+	
+	dir = g_dir_open(path, 0, &error);
+	if (error)
+	{
+		g_warning("Can't list directory:\n%s", error->message);
+		g_error_free(error);
+		return NULL;
+	}
+
+	names = g_ptr_array_new();
+
+	while ((leaf = g_dir_read_name(dir))) {
+		if (leaf[0] != '.')
+			g_ptr_array_add(names, g_strdup(leaf));
+	}
+
+	g_dir_close(dir);
+
+	g_ptr_array_sort(names, strcmp2);
+
+	return names;
+}
+
+int stat_with_timeout(const char *path, struct stat *info)
+{
+	int status;
+	pid_t child;
+	gboolean retval;
+
+	child = fork();
+	if (child < 0)
+	{
+		g_warning("stat_with_timeout: fork(): %s", g_strerror(errno));
+		return -1;
+	}
+
+	if (child == 0)
+	{
+		/* Child */
+		alarm(3);
+		_exit(mc_stat(path, info) ? 1 : 0);
+	}
+
+	waitpid(child, &status, 0);
+
+	if (status == 0)
+		retval = mc_stat(path, info);
+	else
+		retval = -1;
+
+	return retval;
+}
