@@ -1,5 +1,5 @@
 /*
- * $Id: type.c,v 1.133 2003/05/22 13:09:27 tal197 Exp $
+ * $Id: type.c,v 1.153 2004/01/21 20:03:16 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2003, the ROX-Filer team.
@@ -58,6 +58,7 @@
 #include "action.h"		/* (for action_chmod) */
 #include "xml.h"
 #include "dropbox.h"
+#include "gtkicontheme.h"
 
 #define TYPE_NS "http://www.freedesktop.org/standards/shared-mime-info"
 enum {SET_MEDIA, SET_TYPE};
@@ -86,9 +87,12 @@ static GdkColor	type_colours[NUM_TYPE_COLOURS];
 /* Static prototypes */
 static void load_mime_types(void);
 static void alloc_type_colours(void);
+static void options_changed(void);
 static char *get_action_save_path(GtkWidget *dialog);
 static MIME_type *get_mime_type(const gchar *type_name, gboolean can_create);
 static gboolean remove_handler_with_confirm(const guchar *path);
+static void set_icon_theme(void);
+static GList *build_icon_theme(Option *option, xmlNode *node, guchar *label);
 
 /* When working out the type for a file, this hash table is checked
  * first...
@@ -119,6 +123,7 @@ static GHashTable *type_hash = NULL;
 /* Most things on Unix are text files, so this is the default type */
 MIME_type *text_plain;
 MIME_type *inode_directory;
+MIME_type *inode_mountpoint;
 MIME_type *inode_pipe;
 MIME_type *inode_socket;
 MIME_type *inode_block_dev;
@@ -128,16 +133,22 @@ MIME_type *inode_unknown;
 MIME_type *inode_door;
 
 static Option o_display_colour_types;
+static Option o_icon_theme;
+
+static RoxIconTheme *icon_theme = NULL;
 
 void type_init(void)
 {
-	int		i;
+	int	    i;
+
+	icon_theme = rox_icon_theme_new();
 	
 	extension_hash = g_hash_table_new(g_str_hash, g_str_equal);
 	type_hash = g_hash_table_new(g_str_hash, g_str_equal);
 
 	text_plain = get_mime_type("text/plain", TRUE);
 	inode_directory = get_mime_type("inode/directory", TRUE);
+	inode_mountpoint = get_mime_type("inode/mount-point", TRUE);
 	inode_pipe = get_mime_type("inode/fifo", TRUE);
 	inode_socket = get_mime_type("inode/socket", TRUE);
 	inode_block_dev = get_mime_type("inode/blockdevice", TRUE);
@@ -148,7 +159,9 @@ void type_init(void)
 
 	load_mime_types();
 
+	option_add_string(&o_icon_theme, "icon_theme", "ROX");
 	option_add_int(&o_display_colour_types, "display_colour_types", TRUE);
+	option_register_widget("icon-theme-chooser", build_icon_theme);
 	
 	for (i = 0; i < NUM_TYPE_COLOURS; i++)
 		option_add_string(&o_type_colours[i],
@@ -156,7 +169,9 @@ void type_init(void)
 				  opt_type_colours[i][1]);
 	alloc_type_colours();
 
-	option_add_notify(alloc_type_colours);
+	set_icon_theme();
+
+	option_add_notify(options_changed);
 }
 
 /* Read-load all the glob patterns.
@@ -164,6 +179,7 @@ void type_init(void)
  */
 void reread_mime_files(void)
 {
+	rox_icon_theme_rescan_if_needed(icon_theme);
 	load_mime_types();
 }
 
@@ -226,6 +242,26 @@ const char *basetype_name(DirItem *item)
 	return _("Unknown");
 }
 
+static void append_names(gpointer key, gpointer value, gpointer udata)
+{
+	GList **list = (GList **) udata;
+
+	*list = g_list_prepend(*list, key);
+}
+
+/* Return list of all mime type names. Caller must free the list
+ * but NOT the strings it contains (which are never freed).
+ */
+GList *mime_type_name_list(void)
+{
+	GList *list = NULL;
+
+	g_hash_table_foreach(type_hash, append_names, &list);
+	list = g_list_sort(list, (GCompareFunc) strcmp);
+
+	return list;
+}
+
 /*			MIME-type guessing 			*/
 
 /* Get the type of this file - stats the file and uses that if
@@ -233,28 +269,22 @@ const char *basetype_name(DirItem *item)
  */
 MIME_type *type_get_type(const guchar *path)
 {
-	struct stat	info;
+	DirItem		*item;
 	MIME_type	*type = NULL;
-	int		base = TYPE_FILE;
-	gboolean	exec = FALSE;
 
-	if (mc_stat(path, &info) == 0)
-	{
-		base = mode_to_base_type(info.st_mode);
-		if (info.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
-			exec = TRUE;
-	}
+	item = diritem_new("");
+	diritem_restat(path, item, NULL);
+	if (item->base_type != TYPE_ERROR)
+		type = item->mime_type;
+	diritem_free(item);
 
-	if (base == TYPE_FILE)
-		type = type_from_path(path);
+	if (type)
+		return type;
+
+	type = type_from_path(path);
 
 	if (!type)
-	{
-		if (base == TYPE_FILE && exec)
-			type = application_executable;
-		else
-			type = mime_type_from_base_type(base);
-	}
+		return text_plain;
 
 	return type;
 }
@@ -339,6 +369,11 @@ static char *handler_for(MIME_type *type)
 	return open;
 }
 
+MIME_type *mime_type_lookup(const char *type)
+{
+	return get_mime_type(type, TRUE);
+}
+
 /*			Actions for types 			*/
 
 gboolean type_open(const char *path, MIME_type *type)
@@ -391,7 +426,7 @@ gboolean type_open(const char *path, MIME_type *type)
 	else
 		argv[0] = open;
 
-	retval = rox_spawn(home_dir, (const gchar **) argv);
+	retval = rox_spawn(home_dir, (const gchar **) argv) != 0;
 
 	if (argv[0] != open)
 		g_free(argv[0]);
@@ -403,16 +438,17 @@ gboolean type_open(const char *path, MIME_type *type)
 
 /* Return the image for this type, loading it if needed.
  * Places to check are: (eg type="text_plain", base="text")
- * 1. Choices:MIME-icons/<type>
- * 2. Choices:MIME-icons/<base>
- * 3. Unknown type icon.
+ * 1. <Choices>/MIME-icons/base_subtype
+ * 2. Icon theme 'mime-base:subtype'
+ * 3. Icon theme 'mime-base'
+ * 4. Unknown type icon.
  *
  * Note: You must g_object_unref() the image afterwards.
  */
 MaskedPixmap *type_to_icon(MIME_type *type)
 {
-	char	*path;
-	char	*type_name;
+	GdkPixbuf *full;
+	char	*type_name, *path;
 	time_t	now;
 
 	if (type == NULL)
@@ -435,23 +471,50 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 		type->image = NULL;
 	}
 
-	type_name = g_strconcat(type->media_type, "_",
-				type->subtype, ".png", NULL);
+	type_name = g_strconcat(type->media_type, "_", type->subtype,
+				".png", NULL);
 	path = choices_find_path_load(type_name, "MIME-icons");
-	if (!path)
-	{
-		strcpy(type_name + strlen(type->media_type), ".png");
-		path = choices_find_path_load(type_name, "MIME-icons");
-	}
-	
 	g_free(type_name);
-
 	if (path)
 	{
 		type->image = g_fscache_lookup(pixmap_cache, path);
 		g_free(path);
 	}
 
+	if (type->image)
+		goto out;
+
+	type_name = g_strconcat("mime-", type->media_type, ":",
+				type->subtype, NULL);
+	full = rox_icon_theme_load_icon(icon_theme, type_name, HUGE_HEIGHT,
+						0, NULL);
+	g_free(type_name);
+	if (!full)
+	{
+		/* Ugly hack... try for a GNOME icon */
+		type_name = g_strconcat("gnome-mime-", type->media_type,
+				"-", type->subtype, NULL);
+		full = rox_icon_theme_load_icon(icon_theme,
+						type_name,
+						HUGE_HEIGHT, 0, NULL);
+		g_free(type_name);
+	}
+	if (!full)
+	{
+		/* Try for a media type */
+		type_name = g_strconcat("mime-", type->media_type, NULL);
+		full = rox_icon_theme_load_icon(icon_theme,
+						type_name,
+						HUGE_HEIGHT, 0, NULL);
+		g_free(type_name);
+	}
+	if (full)
+	{
+		type->image = masked_pixmap_new(full);
+		g_object_unref(full);
+	}
+
+out:
 	if (!type->image)
 	{
 		/* One ref from the type structure, one returned */
@@ -1070,6 +1133,24 @@ static void alloc_type_colours(void)
 	}
 }
 
+static void expire_timer(gpointer key, gpointer value, gpointer data)
+{
+	MIME_type *type = value;
+
+	type->image_time = 0;
+}
+
+static void options_changed(void)
+{
+	alloc_type_colours();
+	if (o_icon_theme.has_changed)
+	{
+		set_icon_theme();
+		g_hash_table_foreach(type_hash, expire_timer, NULL);
+		full_refresh();
+	}
+}
+
 /* Return a pointer to a (static) colour for this item. If colouring is
  * off, returns normal.
  */
@@ -1289,7 +1370,7 @@ static void load_mime_types(void)
 			"types for different files. You should download and "
 			"install the 'shared-mime-info-0.9' package from "
 			"here:\n"
-		"http://www.freedesktop.org/standards/shared-mime-info.html\n\n"
+		"http://www.freedesktop.org/software/shared-mime-info\n\n"
 		"If you have already installed this package, check that the "
 		"permissions allow the files to be read (check "
 		"/usr/local/share/mime/globs or /usr/share/mime/globs)."));
@@ -1298,16 +1379,13 @@ static void load_mime_types(void)
 	filer_update_all();
 }
 
-/* Try to fill in 'type->comment' from this document.
- * Frees 'path'.
- */
-static void get_comment(MIME_type *type, guchar *path)
+/* Try to fill in 'type->comment' from this document */
+static void get_comment(MIME_type *type, const guchar *path)
 {
 	xmlNode *node;
 	XMLwrapper *doc;
 	
 	doc = xml_cache_load(path);
-	g_free(path);
 	if (!doc)
 		return;
 
@@ -1325,13 +1403,11 @@ static void get_comment(MIME_type *type, guchar *path)
 	g_object_unref(doc);
 }
 
-/* Fill in the comment field for this MIME type.
- *
- * g_object_unref() the result.
- */
+/* Fill in the comment field for this MIME type */
 static void find_comment(MIME_type *type)
 {
-	guchar *path;
+	char **dirs;
+	int i, n_dirs;
 
 	if (type->comment)
 	{
@@ -1339,25 +1415,28 @@ static void find_comment(MIME_type *type)
 		type->comment = NULL;
 	}
 
-	path = g_strdup_printf("%s/.mime/%s/%s.xml", home_dir,
-			type->media_type, type->subtype);
-	get_comment(type, path);
-	if (type->comment)
-		return;
+	dirs = get_xdg_data_dirs(&n_dirs);
+	g_return_if_fail(dirs != NULL);
 
-	path = g_strdup_printf("/usr/local/share/mime/%s/%s.xml",
-			type->media_type, type->subtype);
-	get_comment(type, path);
-	if (type->comment)
-		return;
+	for (i = 0; i < n_dirs; i++)
+	{
+		guchar *path;
+		
+		path = g_strdup_printf("%s/mime/%s/%s.xml", dirs[i],
+				type->media_type, type->subtype);
+		get_comment(type, path);
+		g_free(path);
+		if (type->comment)
+			break;
+	}
 
-	path = g_strdup_printf("/usr/share/mime/%s/%s.xml",
-			type->media_type, type->subtype);
-	get_comment(type, path);
-	if (type->comment)
-		return;
+	if (!type->comment)
+		type->comment = g_strdup_printf("%s/%s", type->media_type,
+						type->subtype);
 
-	type->comment = g_strdup(_("No description"));
+	for (i = 0; i < n_dirs; i++)
+		g_free(dirs[i]);
+	g_free(dirs);
 }
 
 const char *mime_type_comment(MIME_type *type)
@@ -1367,3 +1446,186 @@ const char *mime_type_comment(MIME_type *type)
 
 	return type->comment;
 }
+
+static void set_icon_theme(void)
+{
+	RoxIconInfo *info;
+	char *icon_home;
+	const char *theme_name = o_icon_theme.value;
+
+	if (!theme_name || !*theme_name)
+		theme_name = "ROX";
+
+	while (1)
+	{
+		rox_icon_theme_set_custom_theme(icon_theme, theme_name);
+		info = rox_icon_theme_lookup_icon(icon_theme,
+				"mime-application:postscript",
+				ICON_HEIGHT, 0);
+		if (!info)
+		{
+			info = rox_icon_theme_lookup_icon(icon_theme,
+					"gnome-mime-application-postscript",
+					ICON_HEIGHT, 0);
+		}
+		if (info)
+		{
+			rox_icon_info_free(info);
+			return;
+		}
+
+		if (strcmp(theme_name, "ROX") == 0)
+			break;
+
+		delayed_error(_("Icon theme '%s' does not contain MIME icons. "
+				"Using ROX default theme instead."),
+				theme_name);
+		
+		theme_name = "ROX";
+	}
+
+	delayed_error(_("ROX icon theme not found... installing..."));
+
+	icon_home = g_build_filename(home_dir, ".icons", NULL);
+	if (!file_exists(icon_home))
+		mkdir(icon_home, 0755);
+	g_free(icon_home);	
+
+	icon_home = g_build_filename(home_dir, ".icons", "ROX", NULL);
+	if (symlink(make_path(app_dir, "ROX"), icon_home))
+		delayed_error(_("Failed to create symlink '%s':\n%s"),
+				icon_home, g_strerror(errno));
+	g_free(icon_home);
+
+	rox_icon_theme_rescan_if_needed(icon_theme);
+}
+
+static guchar *read_theme(Option *option)
+{
+	GtkOptionMenu *om = GTK_OPTION_MENU(option->widget);
+	GtkLabel *item;
+
+	item = GTK_LABEL(GTK_BIN(om)->child);
+
+	g_return_val_if_fail(item != NULL, g_strdup("ROX"));
+
+	return g_strdup(gtk_label_get_text(item));
+}
+
+static void update_theme(Option *option)
+{
+	GtkOptionMenu *om = GTK_OPTION_MENU(option->widget);
+	GtkWidget *menu;
+	GList *kids, *next;
+	int i = 0;
+
+	menu = gtk_option_menu_get_menu(om);
+
+	kids = gtk_container_get_children(GTK_CONTAINER(menu));
+	for (next = kids; next; next = next->next, i++)
+	{
+		GtkLabel *item = GTK_LABEL(GTK_BIN(next->data)->child);
+		const gchar *label;
+
+		/* The label actually moves from the menu!! */
+		if (!item)
+			item = GTK_LABEL(GTK_BIN(om)->child);
+
+		label = gtk_label_get_text(item);
+
+		g_return_if_fail(label != NULL);
+
+		if (strcmp(label, option->value) == 0)
+			break;
+	}
+	g_list_free(kids);
+	
+	if (next)
+		gtk_option_menu_set_history(om, i);
+	else
+		g_warning("Theme '%s' not found", option->value);
+}
+
+static void add_themes_from_dir(GPtrArray *names, const char *dir)
+{
+	GPtrArray *list;
+	int i;
+
+	if (access(dir, F_OK) != 0)
+		return;
+
+	list = list_dir(dir);
+	g_return_if_fail(list != NULL);
+
+	for (i = 0; i < list->len; i++)
+	{
+		char *index_path;
+
+		index_path = g_build_filename(dir, list->pdata[i],
+						"index.theme", NULL);
+		
+		if (access(index_path, F_OK) == 0)
+			g_ptr_array_add(names, list->pdata[i]);
+		else
+			g_free(list->pdata[i]);
+
+		g_free(index_path);
+	}
+
+	g_ptr_array_free(list, TRUE);
+}
+
+static GList *build_icon_theme(Option *option, xmlNode *node, guchar *label)
+{
+	GtkWidget *button, *menu, *hbox;
+	GPtrArray *names;
+	gchar **theme_dirs = NULL;
+	gint n_dirs = 0;
+	int i;
+
+	g_return_val_if_fail(option != NULL, NULL);
+	g_return_val_if_fail(label != NULL, NULL);
+
+	hbox = gtk_hbox_new(FALSE, 4);
+
+	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(_(label)),
+				FALSE, TRUE, 0);
+
+	button = gtk_option_menu_new();
+	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+
+	menu = gtk_menu_new();
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(button), menu);
+
+	rox_icon_theme_get_search_path(icon_theme, &theme_dirs, &n_dirs);
+	names = g_ptr_array_new();
+	for (i = 0; i < n_dirs; i++)
+		add_themes_from_dir(names, theme_dirs[i]);
+	g_strfreev(theme_dirs);
+
+	g_ptr_array_sort(names, strcmp2);
+
+	for (i = 0; i < names->len; i++)
+	{
+		GtkWidget *item;
+		char *name = names->pdata[i];
+
+		item = gtk_menu_item_new_with_label(name);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+		gtk_widget_show_all(item);
+
+		g_free(name);
+	}
+
+	g_ptr_array_free(names, TRUE);
+
+	option->update_widget = update_theme;
+	option->read_widget = read_theme;
+	option->widget = button;
+
+	gtk_signal_connect_object(GTK_OBJECT(button), "changed",
+			GTK_SIGNAL_FUNC(option_check_widget),
+			(GtkObject *) option);
+
+	return g_list_append(NULL, hbox);
+}
